@@ -7,6 +7,7 @@ from energy_demand.assumptions import fuel_shares
 from energy_demand.initalisations import helpers
 from energy_demand.profiles import hdd_cdd
 from energy_demand.read_write import narrative_related
+from energy_demand.basic import lookup_tables
 
 class Assumptions(object):
     """Assumptions of energy demand model
@@ -17,7 +18,7 @@ class Assumptions(object):
         Base year
     curr_yr : int, default=None
         Current year
-    simulated_yrs : list, default=None
+    sim_yrs : list, default=None
         Simulated years
     paths : dict, default=None
         Paths
@@ -32,24 +33,34 @@ class Assumptions(object):
     """
     def __init__(
             self,
-            submodels_names=None,
+            lookup_enduses=None,
+            lookup_sector_enduses=None,
             base_yr=None,
+            weather_by=None,
             simulation_end_yr=None,
             curr_yr=None,
-            simulated_yrs=None,
+            sim_yrs=None,
             paths=None,
             enduses=None,
             sectors=None,
-            fueltypes=None,
-            fueltypes_nr=None
+            reg_nrs=None
         ):
         """Constructor
         """
-        self.submodels_names = submodels_names
+        self.lookup_enduses = lookup_enduses
+        self.lookup_sector_enduses = lookup_sector_enduses
+
+        self.submodels_names = lookup_tables.basic_lookups()['submodels_names']
+        self.nr_of_submodels = len(self.submodels_names)
+        self.fueltypes = lookup_tables.basic_lookups()['fueltypes']
+        self.fueltypes_nr = lookup_tables.basic_lookups()['fueltypes_nr']
+
         self.base_yr = base_yr
+        self.weather_by = weather_by
+        self.reg_nrs = reg_nrs
         self.simulation_end_yr = simulation_end_yr
         self.curr_yr = curr_yr
-        self.simulated_yrs = simulated_yrs
+        self.sim_yrs = sim_yrs
 
         # ============================================================
         # Spatially modelled variables
@@ -58,23 +69,17 @@ class Assumptions(object):
         # or technologies having a spatial explicit diffusion need
         # to be defined.
         # ============================================================
-        self.spatial_explicit_diffusion = 0 #0: False, 1: True TODO
+        self.spatial_explicit_diffusion = 0 #0: False, 1: True
 
         # Define all variables which are affected by regional diffusion
-        self.spatially_modelled_vars = [] # ['smart_meter_improvement_p']
+        self.spatially_modelled_vars = [] # ['smart_meter_p']
 
         # Define technologies which are affected by spatial explicit diffusion
         self.techs_affected_spatial_f = ['heat_pumps_electricity']
 
         # Max penetration speed
-        self.speed_con_max = 1.5 # 1: uniform distribution >1: regional differences
+        self.speed_con_max = 1 #1.5 # 1: uniform distribution >1: regional differences
 
-        # --------
-        # Demand management of heat pumps
-        # --------
-        self.flat_heat_pump_profile_both = 0        # 0: False, 1: True
-        self.flat_heat_pump_profile_only_water = 0  # Only water
-    
         # ============================================================
         # Model calibration factors
         # ============================================================
@@ -107,9 +112,9 @@ class Assumptions(object):
         # ============================================================
         #   Modelled day related factors
         # ============================================================
-        #        model_yeardays_date : dict
-        #           Contains for the base year for each days
-        #           the information wheter this is a working or holiday
+        #   model_yeardays_date : dict
+        #     Contains for the base year for each days
+        #     the information wheter this is a working or holiday
         # ------------------------------------------------------------
         self.model_yeardays = list(range(365))
 
@@ -159,7 +164,6 @@ class Assumptions(object):
             'bungalow': 0.088}
 
         self.dwtype_distr_fy = {
-
             'yr_until_changed': yr_until_changed_all_things,
 
             'semi_detached': 0.26,
@@ -176,7 +180,6 @@ class Assumptions(object):
             'bungalow': 77}
 
         self.dwtype_floorarea_fy = {
-
             'yr_until_changed': yr_until_changed_all_things,
 
             'semi_detached': 96,
@@ -208,14 +211,12 @@ class Assumptions(object):
         #           effects will be overestimates (i.e. no multi-
         #           collinearity are considered).
         #
-        #       scenario_drivers : dict
-        #           Scenario drivers per enduse
+        #   scenario_drivers : dict
+        #     Scenario drivers per enduse
         # ------------------------------------------------------------
-        self.scenario_drivers = {}
-
-        # TODO: UPDATE DRIVER WITH GVA
-        # --Residential SubModel
         self.scenario_drivers = {
+
+            # --Residential
             'rs_space_heating': ['floorarea', 'hlc'], # Do not use HDD or pop because otherweise double count
             'rs_water_heating': ['population'],
             'rs_lighting': ['population', 'floorarea'],
@@ -225,7 +226,7 @@ class Assumptions(object):
             'rs_consumer_electronics': ['population', 'gva'],
             'rs_home_computing': ['population'],
 
-            # --Service Submodel (Table 5.5a) TODO USE GVA AS DRIVER
+            # --Service
             'ss_space_heating': ['floorarea'],
             'ss_water_heating': ['population'],
             'ss_lighting': ['floorarea'],
@@ -238,7 +239,7 @@ class Assumptions(object):
             'ss_other_gas': ['population'],
             'ss_other_electricity': ['population'],
 
-            # Industry submodule
+            # Industry
             'is_high_temp_process': ['gva'],
             'is_low_temp_process': ['gva'],
             'is_drying_separation': ['gva'],
@@ -252,9 +253,6 @@ class Assumptions(object):
         # ============================================================
         #   Cooling related assumptions
         # ============================================================
-        #
-        #   Parameters related to cooling enduses are defined here.
-        #
         #   assump_cooling_floorarea : int
         #       The percentage of cooled floor space in the base year
         #
@@ -262,70 +260,65 @@ class Assumptions(object):
         #   ----------
         #   Abela, A. et al. (2016). Study on Energy Use by Air
         #   Conditioning. Bre, (June), 31. Retrieved from
-        #   https://www.bre.co.uk/filelibrary/pdf/projects/aircon-energy-use
-        #   /StudyOnEnergyUseByAirConditioningFinalReport.pdf
+        #   https://www.bre.co.uk/filelibrary/pdf/projects/aircon-energy-use/StudyOnEnergyUseByAirConditioningFinalReport.pdf
         # ------------------------------------------------------------
 
-        # See Abela et al. (2016)
-        # Carbon Trust. (2012). Air conditioning. Maximising comfort, minimising energy consumption
+        # See Abela et al. (2016) & Carbon Trust. (2012). Air conditioning. Maximising comfort, minimising energy consumption
         self.cooled_ss_floorarea_by = 0.35
 
         # ============================================================
         # Smart meter related base year assumptions
         # ============================================================
-        #
-        #   Parameters related to smart metering
-        #
         #   smart_meter_p_by : int
         #       The percentage of households with smart meters in by
-        #   smart_meter_diff_params : dict
-        #       Sigmoid diffusion parameter of smater meters
         # ------------------------------------------------------------
         self.smart_meter_assump = {}
-        self.smart_meter_assump['smart_meter_p_by'] = 0.1
+
+        # Currently in 2017 8.6 mio smart meter installed of 27.2 mio households --> 31.6%
+        # https://assets.publishing.service.gov.uk/government/uploads/system/uploads/attachment_data/file/671930/Smart_Meters_2017_update.pdf)
+        # In 2015, 5.8 % percent of all househods had one: https://assets.publishing.service.gov.uk/government/uploads/system/uploads/attachment_data/file/533060/2016_Q1_Smart_Meters_Report.pdf 
+        self.smart_meter_assump['smart_meter_p_by'] = 0.05
 
         # Long term smart meter induced general savings, purley as
         # a result of having a smart meter (e.g. 0.03 --> 3% savings)
         # DECC 2015: Smart Metering Early Learning Project: Synthesis report
         # https://www.gov.uk/government/publications/smart-metering-early-learning-project-and-small-scale-behaviour-trials
         # Reasonable assumption is between 0.03 and 0.01 (DECC 2015)
-        sm_savings = 0.03
-
         self.smart_meter_assump['savings_smart_meter'] = {
 
             # Residential
-            'rs_cold': sm_savings,
-            'rs_cooking': sm_savings,
-            'rs_lighting': sm_savings,
-            'rs_wet': sm_savings,
-            'rs_consumer_electronics': sm_savings,
-            'rs_home_computing': sm_savings,
-            'rs_space_heating': sm_savings,
-            'rs_water_heating': sm_savings,
+            'rs_cold': 0.03,
+            'rs_cooking': 0.03,
+            'rs_lighting': 0.03,
+            'rs_wet': 0.03,
+            'rs_consumer_electronics': 0.03,
+            'rs_home_computing': 0.03,
+            'rs_space_heating': 0.03,
+            'rs_water_heating': 0.03,
 
             # Service
-            'ss_space_heating': sm_savings,
-            'ss_water_heating': sm_savings,
-            'ss_cooling_humidification': sm_savings,
-            'ss_fans': sm_savings,
-            'ss_lighting': sm_savings,
-            'ss_catering': sm_savings,
-            'ss_small_power': sm_savings,
-            'ss_ICT_equipment': sm_savings,
-            'ss_cooled_storage': sm_savings,
-            'ss_other_gas': sm_savings,
-            'ss_other_electricity': sm_savings,
+            'ss_space_heating': 0.03,
+            'ss_water_heating': 0.03,
+            'ss_cooling_humidification': 0.03,
+            'ss_fans': 0.03,
+            'ss_lighting': 0.03,
+            'ss_catering': 0.03,
+            'ss_small_power': 0.03,
+            'ss_ICT_equipment': 0.03,
+            'ss_cooled_storage': 0.03,
+            'ss_other_gas': 0.03,
+            'ss_other_electricity': 0.03,
 
             # Industry submodule
-            'is_high_temp_process': sm_savings,
-            'is_low_temp_process': sm_savings,
-            'is_drying_separation': sm_savings,
-            'is_motors': sm_savings,
-            'is_compressed_air': sm_savings,
-            'is_lighting': sm_savings,
-            'is_space_heating': sm_savings,
-            'is_other': sm_savings,
-            'is_refrigeration': sm_savings}
+            'is_high_temp_process': 0,
+            'is_low_temp_process': 0,
+            'is_drying_separation': 0,
+            'is_motors': 0,
+            'is_compressed_air': 0,
+            'is_lighting': 0,
+            'is_space_heating': 0,
+            'is_other': 0,
+            'is_refrigeration': 0}
 
         # ============================================================
         # Base temperature assumptions
@@ -333,31 +326,25 @@ class Assumptions(object):
         #
         #   Parameters related to smart metering
         #
-        #   rs_t_heating_by : int
+        #   rs_t_heating : int
         #       Residential submodel base temp of heating of base year
         #   rs_t_cooling_by : int
         #       Residential submodel base temp of cooling of base year
-        #   base_temp_diff_params : dict
-        #       Sigmoid temperature diffusion parameters
         #   ...
         #
         #   Note
         #   ----
         #   Because demand for cooling cannot directly be linked to
-        #   calculated cdd, the paramters 'ss_t_cooling_by' is used
+        #   calculated cdd, the paramters 'ss_t_base_cooling' is used
         #   as a calibration factor. By artifiallcy lowering this
         #   parameter, the energy demand assignement over the days
         #   in a year is improved.
         # ------------------------------------------------------------
-        t_bases = {}
-        t_bases['rs_t_heating_by'] = 15.5    #
-        #t_bases['rs_t_cooling_by'] = Not implemented
-
-        t_bases['ss_t_heating_by'] = 15.5    #
-        t_bases['ss_t_cooling_by'] = 5       # Orig: 5
-
-        t_bases['is_t_heating_by'] = 15.5    #
-        #self.t_bases['is_t_cooling_by'] = Not implemented
+        t_bases = {
+            'rs_t_heating': 15.5,
+            'ss_t_heating': 15.5,
+            'ss_t_cooling': 5,
+            'is_t_heating': 15.5}
 
         self.t_bases = DummyClass(t_bases)
 
@@ -375,75 +362,32 @@ class Assumptions(object):
         #       yd calculations
         # ------------------------------------------------------------
         self.enduse_space_heating = [
-            'rs_space_heating', 'ss_space_heating', 'is_space_heating']
+            'rs_space_heating',
+            'ss_space_heating',
+            'is_space_heating']
 
-        self.ss_enduse_space_cooling = ['ss_cooling_humidification']
+        self.ss_enduse_space_cooling = [
+            'ss_cooling_humidification']
 
         # ============================================================
-        # Industry submodel related parameters
+        # Industry related
+        #
+        # High temperature processing (high_temp_ process) dominates
+        # energy consumption in the iron and steel
+        #
+        # ---- Steel production - Enduse: is_high_temp_process, Sector: basic_metals
+        # With industry service switch, the future shares of 'is_temp_high_process'
+        # in sector 'basic_metals' can be set for 'basic_oxygen_furnace',
+        # 'electric_arc_furnace', and 'SNG_furnace' can be specified
+        #
+        # ---- Cement production - Enduse: is_high_temp_process, Sector: non_metallic_mineral_products
+        # Dry kilns, semidry kilns can be set
         # ============================================================
-        #
-        #   Assumptions related to industrial enduses
-        #
-        #
-        # ------------------------------------------------------------
-
-        # --------------------------------------------
-        # heating
-        # -------------------------------------------- 
-        # --> No scenario drivers but driven by switches
-
-        # --------------------------------------------
-        # lighting
-        #
-        # No individual technologies are defined. Only
-        # overall efficiency increase can be implemented
-        #--------------------------------------------
-
-        # --------------------------------------------
-        # high_temp_ process
-        #
-        #       High temperature processing dominates energy consumption in the iron and steel,
-        #       non-ferrous metal, bricks, cement, glass and potteries industries. This includes
-        #          - coke ovens
-        #          - blast furnaces and other furnaces
-        #          - kilns and
-        #          - glass tanks.
-        # High consumption in Chemicals, Non_metallic mineral products, paper, food_production
-        # Fuel use ratio - electric arc furnace over blast furnace steel making in cement sector
-        # BAT - iron & steel - continous/Ingot casting 	Sectoral share - continuous %
-        # --------------------------------------------
 
         # Share of cold rolling in steel manufacturing
-        # *****************
-        self.p_cold_rolling_steel_by = 0.2      # Estimated  based on https://aceroplatea.es/docs/EuropeanSteelFigures_2015.pdf
-        self.eff_cold_rolling_process = 1.8     # 80% more efficient than hot rolling Fruehan et al. (2002)
-        self.eff_hot_rolling_process = 1.0      # 100% assumed efficiency
-
-        # Steel production - Enduse: is_high_temp_process, Sector: basic_metals
-        # *****************
-        # With industry service switch, the future shares
-        # in 'basic_oxygen_furnace', 'electric_arc_furnace', and 'SNG_furnace'
-        # can be specified
-
-        #scrap-based production: electric arc furnace 
-        #direct reduction process: natrual gas based, electric arc furnace
-        #BF-BOF (blast furnace - basix oxgen furnace)
-        #       Example service switch: 
-        #           is_high_temp_process,SNG_furnace,1.0,2050,basic_metals
-
-        # Cement production - Enduse: is_high_temp_process, Sector: non_metallic_mineral_products
-        # *****************
-        # technologies: Dry kilns, semidry kilns
-
-        # CHEMICALs - Enduse: is_high_temp_process, Sector: CHEMICALS
-        # *****************
-        # technologies: Dry & wet kilns
-        # TODO
-        
-        # -------------
-        # Overall efficiency improvement
-        # -------------
+        self.p_cold_rolling_steel_by = 0.2   # Estimated based on https://aceroplatea.es/docs/EuropeanSteelFigures_2015.pdf
+        self.eff_cold_rolling_process = 1.8  # 80% more efficient than hot rolling Fruehan et al. (2002)
+        self.eff_hot_rolling_process = 1.0   # 100% assumed efficiency
 
         # ============================================================
         # Assumption related to heat pump technologies
@@ -458,8 +402,7 @@ class Assumptions(object):
         self.gshp_fraction = 0.1
 
         # Load defined technologies
-        self.technologies, self.tech_list = read_data.read_technologies(
-            paths['path_technologies'])
+        self.technologies, self.tech_list = read_data.read_technologies(paths['path_technologies'])
 
         self.installed_heat_pump_by = tech_related.generate_ashp_gshp_split(
             self.gshp_fraction)
@@ -468,66 +411,36 @@ class Assumptions(object):
         self.technologies, self.tech_list['heating_non_const'], self.heat_pumps = tech_related.generate_heat_pump_from_split(
             self.technologies,
             self.installed_heat_pump_by,
-            fueltypes)
+            self.fueltypes)
 
         # ============================================================
         # Fuel Stock Definition
-        # Provide for every fueltype of an enduse
-        # the share of fuel which is used by technologies for thebase year
-        # ============================================================
-        self.fuel_tech_p_by = fuel_shares.assign_by_fuel_tech_p(
+        # Provide for every fueltype of an enduse the share of fuel
+        # which is used by technologies in the base year
+        # ============================================================$
+        fuel_tech_p_by = fuel_shares.assign_by_fuel_tech_p(
             enduses,
             sectors,
-            fueltypes,
-            fueltypes_nr)
+            self.fueltypes,
+            self.fueltypes_nr)
 
         # ========================================
-        # Get technologies of an enduse
+        # Get technologies of an enduse and sector
         # ========================================
         self.specified_tech_enduse_by = helpers.get_def_techs(
-            self.fuel_tech_p_by)
+            fuel_tech_p_by)
 
-        specified_tech_enduse_by_new = helpers.add_undef_techs(
+        _specified_tech_enduse_by = helpers.add_undef_techs(
             self.heat_pumps,
             self.specified_tech_enduse_by,
-            'rs_space_heating')
-        self.specified_tech_enduse_by = specified_tech_enduse_by_new
-
-        specified_tech_enduse_by_new = helpers.add_undef_techs(
-            self.heat_pumps,
-            self.specified_tech_enduse_by,
-            'ss_space_heating')
-        self.specified_tech_enduse_by = specified_tech_enduse_by_new
-
-        specified_tech_enduse_by_new = helpers.add_undef_techs(
-            self.heat_pumps,
-            self.specified_tech_enduse_by,
-            'is_space_heating')
-        self.specified_tech_enduse_by = specified_tech_enduse_by_new
-
-        # ============================================================
-        # Read in switches
-        # ============================================================
-        self.fuel_switches = read_data.read_fuel_switches(
-            paths['path_fuel_switches'], enduses, fueltypes, self.technologies)
-
-        self.service_switches = read_data.service_switch(
-            paths['path_service_switch'], self.technologies)
-
-        self.capacity_switches = read_data.read_capacity_switch(
-            paths['path_capacity_installation'])
-
-        self.crit_switch_happening = testing_functions.switch_testing(
-            fuel_switches=self.fuel_switches,
-            service_switches=self.service_switches,
-            capacity_switches=self.capacity_switches)
+            self.enduse_space_heating)
+        self.specified_tech_enduse_by = _specified_tech_enduse_by
 
         # ========================================
-        # General other assumptions
+        # General other info
         # ========================================
         self.seasons = date_prop.get_season(year_to_model=base_yr)
-
-        self.model_yeardays_daytype, self.yeardays_month, self.yeardays_month_days = date_prop.get_model_yeardays_daytype(
+        self.model_yeardays_daytype, self.yeardays_month, self.yeardays_month_days = date_prop.get_yeardays_daytype(
             year_to_model=base_yr)
 
         # ========================================
@@ -535,7 +448,7 @@ class Assumptions(object):
         # ========================================
         self.fuel_tech_p_by, self.specified_tech_enduse_by, self.technologies = tech_related.insert_placholder_techs(
             self.technologies,
-            self.fuel_tech_p_by,
+            fuel_tech_p_by,
             self.specified_tech_enduse_by)
 
         # ========================================
@@ -577,7 +490,7 @@ class Assumptions(object):
 def update_technology_assumption(
         technologies,
         narrative_f_eff_achieved,
-        narrative_gshp_fraction_ey,
+        narrative_gshp_fraction,
         crit_narrative_input=True
     ):
     """Updates technology related properties based on
@@ -592,7 +505,7 @@ def update_technology_assumption(
         Technologies
     f_eff_achieved : float
         Factor achieved
-    gshp_fraction_ey : float
+    gshp_fraction : float
         Mix of GSHP and GSHP
     crit_narrative_input : bool
         Criteria wheter inputs are single values or a narrative
@@ -604,10 +517,10 @@ def update_technology_assumption(
     if crit_narrative_input:
         # Read from narrative the value
         f_eff_achieved = narrative_related.read_from_narrative(narrative_f_eff_achieved) 
-        gshp_fraction_ey = narrative_related.read_from_narrative(narrative_gshp_fraction_ey)
+        gshp_fraction = narrative_related.read_from_narrative(narrative_gshp_fraction)
     else:
         f_eff_achieved = narrative_f_eff_achieved
-        gshp_fraction_ey = narrative_gshp_fraction_ey
+        gshp_fraction = narrative_gshp_fraction
 
     # Assign same achieved efficiency factor for all technologies
     technologies = helpers.set_same_eff_all_tech(
@@ -616,7 +529,7 @@ def update_technology_assumption(
 
     # Calculate average eff of hp depending on fraction of GSHP to ASHP
     installed_heat_pump_ey = tech_related.generate_ashp_gshp_split(
-        gshp_fraction_ey)
+        gshp_fraction)
 
     technologies = tech_related.calc_av_heat_pump_eff_ey(
         technologies, installed_heat_pump_ey)
