@@ -5,68 +5,7 @@ from energy_demand.basic import lookup_tables
 
 from energy_demand.technologies import diffusion_technologies
 
-'''def createNEWCASTLE_dwelling_stock(curr_yr, region, data, parameter_list):
-    """Create dwelling stock based on input from
-    building model from Newcastle
 
-    Arguments
-    ---------
-    curr_yr : int
-        Current year
-    """
-    # ------
-    # Get all regional information concering floor area etc.
-    # Either preprocessed or direct
-    # ------
-    stock_pop = {}
-    floor_area = {}
-    # Iterate all buildings
-    # see which dwelling type -->
-    # see which age_class -->
-    # see which building attribute (e.g. resid, industry)
-    # --> summen the following attributes according these categories
-    #       - population stock_pop[]
-    #       - floor area
-    #       -
-    # ----------------
-    # Create residential dwelling stock
-    # ----------------
-    # Create dwellings Residential
-    # Inputs Residential
-    #   - building_type/dwelling_type/dwellingtype_ageclass
-    #   -
-    dw_stock = []
-    dwelling_types = ["detached", "semi_detached"]
-    age_classs = [1920, 1930, 1940] #Age categories
-    building_type = "residential"
-
-    for dwelling_type in dwelling_types:
-        for age_class in age_classs:
-
-            # Get pop of age class
-            pop_dwtype_age_class = stock_pop[building_type][dwelling_type][age_class]
-
-            # Get floor area of building type, dwelling_ype, age_class
-            floor_area_dwtype_age_class = floor_area[building_type][dwelling_type][age_class]
-
-            dwelling_obj = Dwelling(
-                curr_yr=curr_yr,
-                coordinates=data['reg_coord'][region],
-                floorarea=floor_area_dwtype_age_class,
-                enduses=data['enduses']['residential'],
-                driver_assumptions=data['assumptions'].scenario_drivers['rs_submodule'],
-                population=pop_dwtype_age_class,
-                age=age_class,
-                dwtype=dwelling_type,
-                gva=data['scenario_data']['gva'][curr_yr][region])
-            dw_stock.append(dwelling_obj)
-
-    dwelling_stock = DwellingStock(
-        dw_stock,
-        data['enduses']['residential'])
-
-    return dwelling_stock
-'''
 class Dwelling(object):
     """Dwelling or aggregated group of dwellings
 
@@ -108,8 +47,7 @@ class Dwelling(object):
             age=None,
             dwtype=None,
             sector=None,
-            gva=None,
-            #air_leakage_rate=1.0
+            gva=None
         ):
         """Constructor of Dwelling Class
         """
@@ -122,14 +60,7 @@ class Dwelling(object):
         self.population = population
         self.floorarea = floorarea
         self.sector = sector
-        self.gva = gva #TODO IS THIS USED?
-        #self.air_leakage_rate = air_leakage_rate
-        #self.income = get_income_factor(income)?? MAYBE
-        #self.household_size = get_household_factor(household_size)?? MAYBE
-
-        # FACTORS
-        # HOW MUCH MORE ENERGY e.g. certain dwelling type uses
-        #self.ed_dw_type_factor =
+        self.gva = gva
 
         #: Calculate heat loss coefficient with age and dwelling type if possible
         self.hlc = get_hlc(dwtype, age)
@@ -545,7 +476,7 @@ def rs_dw_stock(
         region,
         assumptions,
         scenario_data,
-        simulated_yrs,
+        sim_yrs,
         dwelling_types,
         enduses,
         reg_coord,
@@ -593,21 +524,21 @@ def rs_dw_stock(
             assumptions.dwtype_floorarea_by,
             assumptions.dwtype_floorarea_fy,
             base_yr,
-            simulated_yrs)
+            sim_yrs)
 
         # Get distribution of dwelling types of all simulation years
         dwtype_distr = get_dwtype_distr(
             assumptions.dwtype_distr_by,
             assumptions.dwtype_distr_fy,
             base_yr,
-            simulated_yrs)
+            sim_yrs)
 
         # Get floor area per person for every simulation year
         data_floorarea_pp = get_floorare_pp(
             scenario_data['floor_area']['rs_floorarea'][base_yr][region],
             scenario_data['population'][base_yr][region],
             base_yr,
-            simulated_yrs,
+            sim_yrs,
             assumptions.non_regional_vars['assump_diff_floorarea_pp'])
 
         # Get fraction of total floorarea for every dwelling type
@@ -820,10 +751,10 @@ def generate_dw_existing(
         dwtype_floorarea = floorarea_p[dwtype_name] * floorarea_by
 
         # Distribute according to age
-        for dwtype_age in dwtype_age_distr_by.keys():
+        for dwtype_age, distribution in dwtype_age_distr_by.items():
 
             # Floor area of dwelling_class_age (distribute proportionally floor area)
-            dwtype_age_class_floorarea = dwtype_floorarea * dwtype_age_distr_by[dwtype_age]
+            dwtype_age_class_floorarea = dwtype_floorarea * distribution
 
             # Floor area per person is divided by base area value to calc pop
             if floorarea_pp != 0:
@@ -965,3 +896,66 @@ def get_hlc(dw_type, age):
         hlc = linear_fits_hlc[dw_type][0] * age + linear_fits_hlc[dw_type][1]
 
         return hlc
+
+'''def createNEWCASTLE_dwelling_stock(curr_yr, region, data, parameter_list):
+    """Create dwelling stock based on input from
+    building model from Newcastle
+
+    Arguments
+    ---------
+    curr_yr : int
+        Current year
+    """
+    # ------
+    # Get all regional information concering floor area etc.
+    # Either preprocessed or direct
+    # ------
+    stock_pop = {}
+    floor_area = {}
+    # Iterate all buildings
+    # see which dwelling type -->
+    # see which age_class -->
+    # see which building attribute (e.g. resid, industry)
+    # --> summen the following attributes according these categories
+    #       - population stock_pop[]
+    #       - floor area
+    #       -
+    # ----------------
+    # Create residential dwelling stock
+    # ----------------
+    # Create dwellings Residential
+    # Inputs Residential
+    #   - building_type/dwelling_type/dwellingtype_ageclass
+    #   -
+    dw_stock = []
+    dwelling_types = ["detached", "semi_detached"]
+    age_classs = [1920, 1930, 1940] #Age categories
+    building_type = "residential"
+
+    for dwelling_type in dwelling_types:
+        for age_class in age_classs:
+
+            # Get pop of age class
+            pop_dwtype_age_class = stock_pop[building_type][dwelling_type][age_class]
+
+            # Get floor area of building type, dwelling_ype, age_class
+            floor_area_dwtype_age_class = floor_area[building_type][dwelling_type][age_class]
+
+            dwelling_obj = Dwelling(
+                curr_yr=curr_yr,
+                coordinates=data['reg_coord'][region],
+                floorarea=floor_area_dwtype_age_class,
+                enduses=data['enduses']['residential'],
+                driver_assumptions=data['assumptions'].scenario_drivers['rs_submodule'],
+                population=pop_dwtype_age_class,
+                age=age_class,
+                dwtype=dwelling_type,
+                gva=data['scenario_data']['gva'][curr_yr][region])
+            dw_stock.append(dwelling_obj)
+
+    dwelling_stock = DwellingStock(
+        dw_stock,
+        data['enduses']['residential'])
+
+    return dwelling_stock
+'''
\ No newline at end of file
