@@ -11,6 +11,7 @@ from energy_demand.basic import basic_functions
 from energy_demand import enduse_func
 from energy_demand.initalisations import helpers
 from energy_demand.profiles import generic_shapes
+from energy_demand.basic import lookup_tables
 
 class WeatherRegion(object):
     """WeaterRegion
@@ -19,12 +20,12 @@ class WeatherRegion(object):
     ----------
     name : str
         Unique identifyer of weather region
+    latitude, longitude : float
+        Coordinates
     assumptions : dict
         Assumptions
     technologies : list
         All technology assumptions
-    fueltypes : dict
-        fueltypes
     enduses : list
         All enduses
     temp_by, temp_ey : array
@@ -42,33 +43,30 @@ class WeatherRegion(object):
     def __init__(
             self,
             name,
+            latitude,
+            longitude,
             assumptions,
             technologies,
-            fueltypes,
             enduses,
             temp_by,
+            temp_cy,
             tech_lp,
-            sectors
+            sectors,
+            crit_temp_min_max
         ):
         """Constructor of weather region
         """
         self.name = name
+        self.longitude = longitude
+        self.latitude = latitude
 
-        # -----------------------------------
-        # Calculate current year temperatures
-        # # INFO: this could be removed if external weather generator is available
-        # -----------------------------------
-        temp_cy = change_temp_climate(
-            assumptions.curr_yr,
-            temp_by,
-            assumptions.yeardays_month_days,
-            assumptions.non_regional_vars)
+        fueltypes = lookup_tables.basic_lookups()['fueltypes']
 
         # Base temperatures of current year
-        rs_t_base_heating_cy = assumptions.non_regional_vars['rs_t_base_heating_future_yr'][assumptions.curr_yr]
-        ss_t_base_heating_cy = assumptions.non_regional_vars['ss_t_base_heating_future_yr'][assumptions.curr_yr]
-        ss_t_base_cooling_cy = assumptions.non_regional_vars['ss_t_base_cooling_future_yr'][assumptions.curr_yr]
-        is_t_base_heating_cy = assumptions.non_regional_vars['is_t_base_heating_future_yr'][assumptions.curr_yr]
+        rs_t_base_heating_cy = assumptions.non_regional_vars['rs_t_base_heating'][assumptions.curr_yr]
+        ss_t_base_heating_cy = assumptions.non_regional_vars['ss_t_base_heating'][assumptions.curr_yr]
+        ss_t_base_cooling_cy = assumptions.non_regional_vars['ss_t_base_cooling'][assumptions.curr_yr]
+        is_t_base_heating_cy = assumptions.non_regional_vars['is_t_base_heating'][assumptions.curr_yr]
 
         # ==================================================================
         # Technology stocks
@@ -81,10 +79,10 @@ class WeatherRegion(object):
             fueltypes,
             temp_by,
             temp_cy,
-            assumptions.t_bases.rs_t_heating_by,
+            assumptions.t_bases.rs_t_heating,
             enduses['residential'],
             rs_t_base_heating_cy,
-            assumptions.specified_tech_enduse_by) #assumptions.rs_specified_tech_enduse_by)
+            assumptions.specified_tech_enduse_by)
 
         ss_tech_stock = technological_stock.TechStock(
             'ss_tech_stock',
@@ -94,10 +92,10 @@ class WeatherRegion(object):
             fueltypes,
             temp_by,
             temp_cy,
-            assumptions.t_bases.ss_t_heating_by,
+            assumptions.t_bases.ss_t_heating,
             enduses['service'],
             ss_t_base_heating_cy,
-            assumptions.specified_tech_enduse_by) #assumptions.ss_specified_tech_enduse_by)
+            assumptions.specified_tech_enduse_by)
 
         is_tech_stock = technological_stock.TechStock(
             'is_tech_stock',
@@ -107,31 +105,34 @@ class WeatherRegion(object):
             fueltypes,
             temp_by,
             temp_cy,
-            assumptions.t_bases.is_t_heating_by,
+            assumptions.t_bases.is_t_heating,
             enduses['industry'],
             ss_t_base_heating_cy,
-            assumptions.specified_tech_enduse_by) #assumptions.is_specified_tech_enduse_by)
-        
+            assumptions.specified_tech_enduse_by)
+
         self.tech_stock = {
             'residential': rs_tech_stock,
             'service': ss_tech_stock,
-            'industry': is_tech_stock
-        }
+            'industry': is_tech_stock}
+
         # ==================================================================
         # Load profiles
         # ==================================================================
         flat_shape_yd, _, flat_shape_y_dh = generic_shapes.flat_shape()
-       
+
         # ==================================================================
         # Residential submodel load profiles
         # ==================================================================
         load_profiles = load_profile.LoadProfileStock("load_profiles")
+        dummy_sector = None
 
-        # --------Calculate HDD/CDD
+        # --------Calculate HDD/CDD of used weather year
         self.rs_hdd_by, _ = hdd_cdd.calc_reg_hdd(
-            temp_by, assumptions.t_bases.rs_t_heating_by, assumptions.model_yeardays)
+            temp_by, assumptions.t_bases.rs_t_heating, assumptions.model_yeardays, crit_temp_min_max)
         self.rs_hdd_cy, rs_fuel_shape_heating_yd = hdd_cdd.calc_reg_hdd(
-            temp_cy, rs_t_base_heating_cy, assumptions.model_yeardays)
+            temp_cy, rs_t_base_heating_cy, assumptions.model_yeardays, crit_temp_min_max)
+
+        # --------Calculate HDD/CDD of base year weather year
         #self.rs_cdd_by, _ = hdd_cdd.calc_reg_cdd(
         #    temp_by, assumptions.t_bases.rs_t_cooling_by, assumptions.model_yeardays)
         #self.rs_cdd_cy, rs_fuel_shape_cooling_yd = hdd_cdd.calc_reg_cdd(
@@ -161,10 +162,9 @@ class WeatherRegion(object):
             if enduse in ['rs_space_heating']:
                 pass
             else:
-
                 # Get all technologies of enduse
                 tech_list = helpers.get_nested_dict_key(
-                    assumptions.fuel_tech_p_by[enduse])
+                    assumptions.fuel_tech_p_by[enduse][dummy_sector])
 
                 # Remove heat pumps from rs_water_heating
                 tech_list = basic_functions.remove_element_from_list(tech_list, 'heat_pumps_electricity')
@@ -179,13 +179,13 @@ class WeatherRegion(object):
                     technologies=tech_list,
                     enduses=[enduse],
                     shape_yd=tech_lp['rs_shapes_yd'][enduse]['shape_non_peak_yd'],
-                    shape_y_dh=shape_y_dh, #tech_lp['rs_shapes_dh'][enduse]['shape_non_peak_y_dh'],
+                    shape_y_dh=shape_y_dh,
+                    sectors=[dummy_sector],
                     model_yeardays=assumptions.model_yeardays)
 
         # ==========
         # Technology specific profiles for residential heating
         # ===========
-
         # ------Heating boiler
         load_profiles.add_lp(
             unique_identifier=uuid.uuid4(),
@@ -193,6 +193,7 @@ class WeatherRegion(object):
             enduses=['rs_space_heating'],
             shape_yd=rs_fuel_shape_heating_yd,
             shape_y_dh=tech_lp['rs_profile_boilers_y_dh'],
+            sectors=[dummy_sector],
             model_yeardays=assumptions.model_yeardays)
 
         # ------Heating CHP
@@ -206,7 +207,8 @@ class WeatherRegion(object):
             technologies=assumptions.tech_list['tech_CHP'],
             enduses=['rs_space_heating'],
             shape_yd=rs_fuel_shape_heating_yd,
-            shape_y_dh=rs_profile_chp_y_dh, #tech_lp['rs_profile_chp_y_dh']
+            shape_y_dh=rs_profile_chp_y_dh,
+            sectors=[dummy_sector],
             model_yeardays=assumptions.model_yeardays)
 
         # ------Electric heating, storage heating (primary)
@@ -216,6 +218,7 @@ class WeatherRegion(object):
             enduses=['rs_space_heating'],
             shape_yd=rs_fuel_shape_heating_yd,
             shape_y_dh=tech_lp['rs_profile_storage_heater_y_dh'],
+            sectors=[dummy_sector],
             model_yeardays=assumptions.model_yeardays)
 
         # ------Electric heating secondary (direct elec heating)
@@ -225,6 +228,7 @@ class WeatherRegion(object):
             enduses=['rs_space_heating'],
             shape_yd=rs_fuel_shape_heating_yd,
             shape_y_dh=tech_lp['rs_profile_elec_heater_y_dh'],
+            sectors=[dummy_sector],
             model_yeardays=assumptions.model_yeardays)
 
         # ------Heat pump heating
@@ -234,98 +238,53 @@ class WeatherRegion(object):
             shape_peak_dh=tech_lp['rs_lp_heating_hp_dh']['peakday'])
 
         rs_fuel_shape_hp_yh, rs_hp_shape_yd = get_fuel_shape_heating_hp_yh(
-            tech_lp_y_dh=rs_profile_hp_y_dh, #tech_lp['rs_profile_hp_y_dh'], #
+            tech_lp_y_dh=rs_profile_hp_y_dh,
             tech_stock=rs_tech_stock,
             rs_hdd_cy=self.rs_hdd_cy,
             model_yeardays=assumptions.model_yeardays)
 
-        # Flat lp
-        flat_rs_fuel_shape_hp_yh, rs_hp_shape_yd = get_fuel_shape_heating_hp_yh(
-            tech_lp_y_dh=flat_shape_y_dh,
-            tech_stock=rs_tech_stock,
-            rs_hdd_cy=self.rs_hdd_cy,
+        load_profiles.add_lp(
+            unique_identifier=uuid.uuid4(),
+            technologies=assumptions.tech_list['heating_non_const'],
+            enduses=['rs_space_heating', 'rs_water_heating'],
+            shape_y_dh=rs_profile_hp_y_dh,
+            shape_yd=rs_hp_shape_yd,
+            shape_yh=rs_fuel_shape_hp_yh,
+            sectors=[dummy_sector],
             model_yeardays=assumptions.model_yeardays)
 
-        # ------------
-        # Set criteria wheter to use float load profile for heat pumps or not
-        # ------------
-        if assumptions.non_regional_vars['flat_heat_pump_profile_both']:
-            flat_space_heating = True
-            flat_water_heating = True
-        elif assumptions.non_regional_vars['flat_heat_pump_profile_only_water']:
-            flat_space_heating = False
-            flat_water_heating = True
-        else:
-            flat_space_heating = False
-            flat_water_heating = False
-
-        if flat_water_heating and flat_space_heating:
-            # Flat load profiles for water and space heating
-            load_profiles.add_lp(
-                unique_identifier=uuid.uuid4(),
-                technologies=assumptions.tech_list['heating_non_const'],
-                enduses=['rs_space_heating', 'rs_water_heating'],
-                shape_y_dh=flat_shape_y_dh,
-                shape_yd=rs_hp_shape_yd,
-                shape_yh=flat_rs_fuel_shape_hp_yh,
-                model_yeardays=assumptions.model_yeardays)
-        elif flat_water_heating and not flat_space_heating:
-            # Flat load profiles for water heating
-            load_profiles.add_lp(
-                unique_identifier=uuid.uuid4(),
-                technologies=assumptions.tech_list['heating_non_const'],
-                enduses=['rs_water_heating'],
-                shape_y_dh=flat_shape_y_dh,
-                shape_yd=rs_hp_shape_yd,
-                shape_yh=flat_rs_fuel_shape_hp_yh,
-                model_yeardays=assumptions.model_yeardays)
-            load_profiles.add_lp(
-                unique_identifier=uuid.uuid4(),
-                technologies=assumptions.tech_list['heating_non_const'],
-                enduses=['rs_space_heating'],
-                shape_y_dh=rs_profile_hp_y_dh, #tech_lp['rs_profile_hp_y_dh'],
-                shape_yd=rs_hp_shape_yd,
-                shape_yh=rs_fuel_shape_hp_yh,
-                model_yeardays=assumptions.model_yeardays)
-        else:
-            # No flat load profile
-            load_profiles.add_lp(
-                unique_identifier=uuid.uuid4(),
-                technologies=assumptions.tech_list['heating_non_const'],
-                enduses=['rs_space_heating', 'rs_water_heating'],
-                shape_y_dh=tech_lp['rs_profile_hp_y_dh'],
-                shape_yd=rs_hp_shape_yd,
-                shape_yh=rs_fuel_shape_hp_yh,
-                model_yeardays=assumptions.model_yeardays)
-
         # ------District_heating_electricity. Assumption made that same curve as CHP
         load_profiles.add_lp(
             unique_identifier=uuid.uuid4(),
             technologies=assumptions.tech_list['tech_district_heating'],
             enduses=['rs_space_heating'],
             shape_yd=rs_fuel_shape_heating_yd,
-            shape_y_dh=rs_profile_hp_y_dh, #tech_lp['rs_profile_chp_y_dh'],
+            shape_y_dh=rs_profile_hp_y_dh,
+            sectors=[dummy_sector],
             model_yeardays=assumptions.model_yeardays)
 
         # ==================================================================
         # Service Submodel load profiles
         # ==================================================================
         # --------HDD/CDD
-        ss_hdd_by, _ = hdd_cdd.calc_reg_hdd(
-            temp_by, assumptions.t_bases.ss_t_heating_by, assumptions.model_yeardays)
+        # current weather_yr
+        self.ss_hdd_by, _ = hdd_cdd.calc_reg_hdd(
+            temp_by, assumptions.t_bases.ss_t_heating, assumptions.model_yeardays, crit_temp_min_max)
+
         ss_hdd_cy, ss_fuel_shape_heating_yd = hdd_cdd.calc_reg_hdd(
-            temp_cy, ss_t_base_heating_cy, assumptions.model_yeardays)
+            temp_cy, ss_t_base_heating_cy, assumptions.model_yeardays, crit_temp_min_max)
+
+        self.ss_cdd_by, _ = hdd_cdd.calc_reg_cdd(
+            temp_by, assumptions.t_bases.ss_t_cooling, assumptions.model_yeardays, crit_temp_min_max)
 
-        ss_cdd_by, _ = hdd_cdd.calc_reg_cdd(
-            temp_by, assumptions.t_bases.ss_t_cooling_by, assumptions.model_yeardays)
         ss_cdd_cy, ss_lp_cooling_yd = hdd_cdd.calc_reg_cdd(
-            temp_cy, ss_t_base_cooling_cy, assumptions.model_yeardays)
+            temp_cy, ss_t_base_cooling_cy, assumptions.model_yeardays, crit_temp_min_max)
 
         try:
             f_heat_ss_y = np.nan_to_num(
-                1.0 / float(np.sum(ss_hdd_by))) * np.sum(ss_hdd_cy)
+                1.0 / float(np.sum(self.ss_hdd_by))) * np.sum(ss_hdd_cy)
             f_cooling_ss_y = np.nan_to_num(
-                1.0 / float(np.sum(ss_cdd_by))) * np.sum(ss_cdd_cy)
+                1.0 / float(np.sum(self.ss_cdd_by))) * np.sum(ss_cdd_cy)
         except ZeroDivisionError:
             f_heat_ss_y = 1
             f_cooling_ss_y = 1
@@ -377,7 +336,7 @@ class WeatherRegion(object):
             all_techs_ss_space_heating, 'heat_pumps_electricity')
 
         ss_fuel_shape_hp_yh, ss_hp_shape_yd = get_fuel_shape_heating_hp_yh(
-            tech_lp_y_dh=rs_profile_hp_y_dh, #tech_lp['rs_profile_hp_y_dh'],
+            tech_lp_y_dh=rs_profile_hp_y_dh,
             tech_stock=rs_tech_stock,
             rs_hdd_cy=ss_hdd_cy,
             model_yeardays=assumptions.model_yeardays)
@@ -387,7 +346,7 @@ class WeatherRegion(object):
             technologies=assumptions.tech_list['heating_non_const'],
             enduses=['ss_space_heating', 'ss_water_heating'],
             sectors=sectors['service'],
-            shape_y_dh=rs_profile_hp_y_dh, #tech_lp['rs_profile_hp_y_dh'],
+            shape_y_dh=rs_profile_hp_y_dh,
             shape_yd=ss_hp_shape_yd,
             shape_yh=ss_fuel_shape_hp_yh,
             model_yeardays=assumptions.model_yeardays)
@@ -444,19 +403,21 @@ class WeatherRegion(object):
         # ==================================================================
 
         # --------HDD/CDD
-        is_hdd_by, _ = hdd_cdd.calc_reg_hdd(
-            temp_by, assumptions.t_bases.is_t_heating_by, assumptions.model_yeardays)
+        # Current weather_yr
+        self.is_hdd_by, _ = hdd_cdd.calc_reg_hdd(
+            temp_by, assumptions.t_bases.is_t_heating, assumptions.model_yeardays, crit_temp_min_max)
+
         #is_cdd_by, _ = hdd_cdd.calc_reg_cdd(
-        #    temp_by, assumptions.t_bases.is_t_cooling_by, assumptions.model_yeardays)
+        #    temp_by, assumptions.t_bases.is_t_cooling, assumptions.model_yeardays)
 
         # Take same base temperature as for service sector
         is_hdd_cy, is_fuel_shape_heating_yd = hdd_cdd.calc_reg_hdd(
-            temp_cy, is_t_base_heating_cy, assumptions.model_yeardays)
+            temp_cy, is_t_base_heating_cy, assumptions.model_yeardays, crit_temp_min_max)
         #is_cdd_cy, _ = hdd_cdd.calc_reg_cdd(
         #    temp_cy, ss_t_base_cooling_cy, assumptions.model_yeardays)
 
         try:
-            f_heat_is_y = np.nan_to_num(1.0 / float(np.sum(is_hdd_by))) * np.sum(is_hdd_cy)
+            f_heat_is_y = np.nan_to_num(1.0 / float(np.sum(self.is_hdd_by))) * np.sum(is_hdd_cy)
             #self.f_cooling_is_y = np.nan_to_num(1.0 / float(np.sum(is_cdd_by))) * np.sum(is_cdd_cy)
             f_cooling_is_y = 1
         except ZeroDivisionError:
@@ -498,7 +459,7 @@ class WeatherRegion(object):
             enduses=['is_space_heating'],
             sectors=sectors['industry'],
             shape_yd=is_fuel_shape_heating_yd_weighted,
-            shape_y_dh=flat_shape_y_dh, # tech_lp['ss_all_tech_shapes_dh']['ss_space_heating']['shape_non_peak_y_dh]
+            shape_y_dh=flat_shape_y_dh,
             model_yeardays=assumptions.model_yeardays)
 
         # Apply correction factor for weekend_effect to flat load profile for industry
@@ -576,6 +537,7 @@ def get_fuel_shape_heating_hp_yh(tech_lp_y_dh, tech_stock, rs_hdd_cy, model_year
 
     tech_eff = tech_stock.get_tech_attr(
         'rs_space_heating',
+        None,
         'heat_pumps_electricity',
         'eff_cy')
 
@@ -598,7 +560,7 @@ def get_fuel_shape_heating_hp_yh(tech_lp_y_dh, tech_stock, rs_hdd_cy, model_year
     shape_y_dh[np.isnan(shape_y_dh)] = 0
 
     # Select only modelled days
-    return shape_yh[[model_yeardays]], hp_yd
+    return shape_yh[model_yeardays], hp_yd
 
 def get_shape_cooling_yh(tech_shape, cooling_shape):
     """Convert daily shape to hourly
@@ -662,44 +624,6 @@ def ss_get_sector_enduse_shape(tech_lps, heating_lp_yd, enduse):
 
     return shape_yh_generic_tech, shape_y_dh_generic_tech
 
-def change_temp_climate(
-        current_yr,
-        temp_data,
-        yeardays_month_days,
-        non_regional_strategy_variables,
-    ):
-    """Change temperature data for every year depending
-    on simple climate change assumptions
-
-    Arguments
-    ---------
-    temp_data : dict
-        Data
-    yeardays_month_days : dict
-        Month containing all yeardays
-    strategy_variables : dict
-        Assumption on temperature change
-
-    Returns
-    -------
-    temp_climate_change : dict
-        Adapted temperatures for all weather stations depending on climate change assumptions
-    """
-    temp_climate_change = np.zeros((365, 24), dtype="float")
-
-    # Iterate every month
-    for yearday_month, month_yeardays in yeardays_month_days.items():
-
-        month_str = basic_functions.get_month_from_int(yearday_month + 1)
-
-        # Calculate monthly change in temperature
-        change_temp_cy = non_regional_strategy_variables["climate_change_temp_d"][month_str][current_yr]
-
-        # Add change
-        temp_climate_change[month_yeardays] = temp_data[month_yeardays] + change_temp_cy
-
-    return temp_climate_change
-
 def insert_peak_dh_shape(
         peak_day,
         shape_y_dh,
@@ -720,11 +644,38 @@ def insert_peak_dh_shape(
         Array where on peak day the peak shape is inserted
     """
     shape_y_dh_inserted = np.copy(shape_y_dh)
-    #print(np.sum(shape_y_dh_inserted))
 
     shape_y_dh_inserted[peak_day] = shape_peak_dh
 
-    #print(np.sum(shape_y_dh_inserted))
-    #assert np.sum(shape_y_dh_inserted) == 365.0
-
     return shape_y_dh_inserted
+
+def get_weather_station_selection(
+        all_weather_stations,
+        counter,
+        weather_yr
+    ):
+    """Select weather stations based on
+    position in list
+
+    Arguments
+    --------
+    counter : int
+        Count to get weather station nr
+    """
+    all_stations_of_weather_yr = list(all_weather_stations[weather_yr].keys())
+
+    # Sort
+    all_stations_of_weather_yr.sort()
+
+    try:
+        # Select station according ot position in list
+        station_id = all_stations_of_weather_yr[counter]
+
+        all_weather_stations_out = {station_id: all_weather_stations[weather_yr][station_id]}
+    except KeyError:
+        # Not enough stations to select position in list
+        station_id = False
+        all_weather_stations_out = []
+        logging.debug("... no weather station found")
+
+    return all_weather_stations_out, station_id
