@@ -1,20 +1,23 @@
 """
 This file containes functions to plot multiple scenarios in a folder
 """
+import math
 import os
 import operator
 import collections
 import numpy as np
+import matplotlib.pyplot as plt
+import pandas as pd
+
 from energy_demand.plotting import plotting_styles
-from energy_demand.plotting import plotting_program
+from energy_demand.plotting import basic_plot_functions
 from energy_demand.basic import conversions
 from energy_demand.plotting import plotting_results
 from energy_demand import enduse_func
 from energy_demand.profiles import load_factors
 from energy_demand.read_write import write_data
-import matplotlib
-import matplotlib.pyplot as plt
-matplotlib.use('Agg') # Used to make it work in linux
+from energy_demand.basic import date_prop
+#matplotlib.use('Agg') # Used to make it work in linux
 
 def plot_heat_pump_chart_multiple(
         lookups,
@@ -109,7 +112,7 @@ def plot_heat_pump_chart_multiple(
     plot_all_regs = False
 
     # Set figure size
-    fig = plt.figure(figsize=plotting_program.cm2inch(9, 8))
+    fig = plt.figure(figsize=basic_plot_functions.cm2inch(9, 8))
     ax = fig.add_subplot(1, 1, 1)
 
     # -----------------
@@ -318,7 +321,7 @@ def plot_heat_pump_chart(
     plot_all_regs = False
 
     # Set figure size
-    fig = plt.figure(figsize=plotting_program.cm2inch(16, 8))
+    fig = plt.figure(figsize=basic_plot_functions.cm2inch(16, 8))
     ax = fig.add_subplot(1, 1, 1)
 
     # -----------------
@@ -455,7 +458,7 @@ def plot_tot_y_peak_hour(
     ):
     """Plot fueltype specific peak h of all regions
     """
-    plt.figure(figsize=plotting_program.cm2inch(14, 8))
+    plt.figure(figsize=basic_plot_functions.cm2inch(14, 8))
 
     # -----------------
     # Axis
@@ -542,14 +545,14 @@ def plot_reg_y_over_time(
     scenario for all regions
     """
     # Set figure size
-    plt.figure(figsize=plotting_program.cm2inch(14, 8))
+    plt.figure(figsize=basic_plot_functions.cm2inch(14, 8))
 
     y_scenario = {}
 
     for scenario_name, scen_data in scenario_data.items():
 
         data_years_regs = {}
-        for year, fueltype_reg_time in scen_data['results_every_year'].items():
+        for year, fueltype_reg_time in scen_data['ed_fueltype_regs_yh'].items():
             data_years_regs[year] = {}
 
             for _fueltype, regions_fuel in enumerate(fueltype_reg_time):
@@ -658,7 +661,7 @@ def plot_tot_fueltype_y_over_time(
 
     # Set figure size
     fig = plt.figure(
-        figsize=plotting_program.cm2inch(9, 8))
+        figsize=basic_plot_functions.cm2inch(9, 8))
 
     ax = fig.add_subplot(1, 1, 1)
 
@@ -668,7 +671,7 @@ def plot_tot_fueltype_y_over_time(
 
         # Read out fueltype specific max h load
         data_years = {}
-        for year, fueltype_reg_time in scen_data['results_every_year'].items():
+        for year, fueltype_reg_time in scen_data['ed_fueltype_regs_yh'].items():
 
             # Sum all regions
             tot_gwh_fueltype_yh = np.sum(fueltype_reg_time, axis=1)
@@ -796,7 +799,7 @@ def plot_tot_y_over_time(
     scenario for all regions
     """
     # Set figure size
-    plt.figure(figsize=plotting_program.cm2inch(14, 8))
+    plt.figure(figsize=basic_plot_functions.cm2inch(14, 8))
 
     y_scenario = {}
 
@@ -804,7 +807,7 @@ def plot_tot_y_over_time(
 
         # Read out fueltype specific max h load
         data_years = {}
-        for year, fueltype_reg_time in scen_data['results_every_year'].items():
+        for year, fueltype_reg_time in scen_data['ed_fueltype_regs_yh'].items():
 
             # Sum all regions and fueltypes
             tot_gwh_fueltype_y = np.sum(fueltype_reg_time)
@@ -891,36 +894,54 @@ def plot_radar_plots_average_peak_day(
 
     MAYBE: SO FAR ONLY FOR ONE SCENARIO
     """
-    name_spider_plot = os.path.join(
-        fig_name, "spider_scenarios_{}.pdf".format(fueltype_to_model))
-
     fueltype_int = fueltypes[fueltype_to_model]
 
     # ----------------
     # Create base year peak load profile
     # Aggregate load profiles of all regions
     # -----------------
-    individ_radars_to_plot_dh = []
+    peak_winter_individ_radars_to_plot_dh = []
+    trough_summer_individ_radars_to_plot_dh = []
     load_factor_fueltype_y_cy = []
     results_txt = []
 
     for scenario_cnt, scenario in enumerate(scenario_data):
 
-        print("-------Scenario: {} {}".format(scenario, fueltype_to_model))
+        print("-------Scenario: '{}' '{}'".format(scenario, fueltype_to_model))
 
         # ------------------------
         # Future year load profile
         # ------------------------
-        all_regs_fueltypes_yh_by = np.sum(scenario_data[scenario]['results_every_year'][base_yr], axis=1)
-        all_regs_fueltypes_yh_cy = np.sum(scenario_data[scenario]['results_every_year'][year_to_plot], axis=1)
+        all_regs_fueltypes_yh_by = np.sum(scenario_data[scenario]['ed_fueltype_regs_yh'][base_yr], axis=1)
+        all_regs_fueltypes_yh_cy = np.sum(scenario_data[scenario]['ed_fueltype_regs_yh'][year_to_plot], axis=1)
 
         # ---------------------------
         # Calculate load factors
         # ---------------------------
         peak_day_nr_by, by_max_h = enduse_func.get_peak_day_single_fueltype(
             all_regs_fueltypes_yh_by[fueltype_int])
+        test_peak_day_by = enduse_func.get_peak_day_all_fueltypes(all_regs_fueltypes_yh_by)
         peak_day_nr_cy, cy_max_h = enduse_func.get_peak_day_single_fueltype(
             all_regs_fueltypes_yh_cy[fueltype_int])
+        test_peak_day_cy = enduse_func.get_peak_day_all_fueltypes(all_regs_fueltypes_yh_cy)
+
+        #peak_day_nr_cy = 32
+
+        _ = all_regs_fueltypes_yh_cy[fueltype_int].reshape(365,24)
+        print(_[peak_day_nr_cy])
+        print(np.max(_[peak_day_nr_cy]))
+        print(cy_max_h)
+
+        print("-----{}  {}".format(scenario, peak_day_nr_cy))
+        print(_[peak_day_nr_cy])
+        print("EGON")
+
+        # Minimum trough day
+        trough_day_nr_by, by_max_h_trough = enduse_func.get_trough_day_single_fueltype(
+            all_regs_fueltypes_yh_by[fueltype_int])
+
+        trough_day_nr_cy, cy_max_h_trough = enduse_func.get_trough_day_single_fueltype(
+            all_regs_fueltypes_yh_cy[fueltype_int])
 
         scen_load_factor_fueltype_y_by = load_factors.calc_lf_y(all_regs_fueltypes_yh_by)
         load_factor_fueltype_y_by = round(scen_load_factor_fueltype_y_by[fueltype_int], fueltype_int)
@@ -928,6 +949,16 @@ def plot_radar_plots_average_peak_day(
         scen_load_factor_fueltype_y_cy = load_factors.calc_lf_y(all_regs_fueltypes_yh_cy)
         load_factor_fueltype_y_cy.append(round(scen_load_factor_fueltype_y_cy[fueltype_int], fueltype_int))
 
+        # Add info on peak days #by2: {} 
+        results_txt.append("p_day: {} by1: {} cy1: {}: cy2: {} trough_by: {} trough_cy: {}".format(
+            peak_day_nr_cy,
+            date_prop.yearday_to_date(2015, peak_day_nr_by), 
+            date_prop.yearday_to_date(2015, peak_day_nr_cy),
+            #date_prop.yearday_to_date(2015, test_peak_day_by),
+            date_prop.yearday_to_date(2015, test_peak_day_cy),
+            date_prop.yearday_to_date(2015, trough_day_nr_by),
+            date_prop.yearday_to_date(2015, trough_day_nr_cy)))
+
         # ------------------------
         # Restult calculations
         # ------------------------
@@ -939,17 +970,21 @@ def plot_radar_plots_average_peak_day(
             'ss_water_heating',
             'is_space_heating']
 
-        aggregated_enduse_fueltype_cy = np.zeros((365, 24))
-        aggregated_enduse_fueltype_by = np.zeros((365, 24))
+        aggregated_enduse_fueltype_cy = np.zeros((8760))
+        aggregated_enduse_fueltype_by = np.zeros((8760))
 
         for enduse in scenario_data[scenario]['results_enduse_every_year'][year_to_plot].keys():
             if enduse in enduses_to_agg:
                 aggregated_enduse_fueltype_by += scenario_data[scenario]['results_enduse_every_year'][2015][enduse][fueltype_int]
                 aggregated_enduse_fueltype_cy += scenario_data[scenario]['results_enduse_every_year'][year_to_plot][enduse][fueltype_int]
+        aggregated_enduse_fueltype_by_8760 = aggregated_enduse_fueltype_by.reshape(365, 24)
+        aggregated_enduse_fueltype_cy_8760 = aggregated_enduse_fueltype_cy.reshape(365, 24)
 
         # Total demand of selected enduses
-        selected_enduses_peak_by = round(np.max(aggregated_enduse_fueltype_by), 1)
-        selected_enduses_peak_cy = round(np.max(aggregated_enduse_fueltype_cy[peak_day_nr_cy]), 1)
+        selected_enduses_peak_by = round(np.max(aggregated_enduse_fueltype_by_8760[peak_day_nr_by]), 1)
+        selected_enduses_peak_cy = round(np.max(aggregated_enduse_fueltype_cy_8760[peak_day_nr_cy]), 1)
+        selected_enduses_min_by = round(np.max(aggregated_enduse_fueltype_by_8760[trough_day_nr_by]), 1)
+        selected_enduses_min_cy = round(np.max(aggregated_enduse_fueltype_cy_8760[trough_day_nr_cy]), 1)
 
         # Calculate change in peak
         all_regs_fueltypes_yh_by = all_regs_fueltypes_yh_by.reshape(all_regs_fueltypes_yh_by.shape[0], 365, 24)
@@ -957,15 +992,20 @@ def plot_radar_plots_average_peak_day(
 
         diff_max_h = round(((100 / by_max_h) * cy_max_h) - 100, 1)
 
-        txt = "peak_h_by: {} peak_h_cy: {} diff_p: {}, peak_h_selected_enduses_by: {} peak_h_selected_enduses_cy: {} scenario: {}".format(
+        txt = "peak_h_by: {} peak_h_cy: {} winter_heating_by: {} winter_heating_cy: {} trough_by: {} trough_cy: {} summer_heating_by: {} summer_heating_cy: {} {}".format(
             round(by_max_h, 1),
             round(cy_max_h, 1),
-            round(diff_max_h, 1),
             selected_enduses_peak_by,
             selected_enduses_peak_cy,
+            round(by_max_h_trough, 1),
+            round(cy_max_h_trough, 1),
+            selected_enduses_min_by,
+            selected_enduses_min_cy,
             scenario)
-
+    
         results_txt.append(txt)
+        results_txt.append("----------")
+        #results_txt.append("trough:day: " + str(all_regs_fueltypes_yh_by[fueltype_int][trough_day_nr_by]))
 
         print("Calculation of diff in peak: {} {} {} {}".format(
             scenario, round(diff_max_h, 1), round(by_max_h, 1), round(cy_max_h, 1)))
@@ -974,26 +1014,53 @@ def plot_radar_plots_average_peak_day(
         # Plot dh for peak day for base year
         # ----------------------------------
         if scenario_cnt == 0:
-            individ_radars_to_plot_dh.append(list(all_regs_fueltypes_yh_by[fueltype_int][peak_day_nr_by]))
+            peak_winter_individ_radars_to_plot_dh.append(list(all_regs_fueltypes_yh_by[fueltype_int][peak_day_nr_by]))
+            trough_summer_individ_radars_to_plot_dh.append(list(all_regs_fueltypes_yh_by[fueltype_int][trough_day_nr_cy]))
         else:
             pass
 
-        # Add current year
-        individ_radars_to_plot_dh.append(list(all_regs_fueltypes_yh_cy[fueltype_int][peak_day_nr_cy]))
+        # Add current year maximum winter peak day
+        peak_winter_individ_radars_to_plot_dh.append(list(all_regs_fueltypes_yh_cy[fueltype_int][peak_day_nr_cy]))
 
+        # Add current year minimum summer trough day
+        trough_summer_individ_radars_to_plot_dh.append(list(all_regs_fueltypes_yh_cy[fueltype_int][trough_day_nr_cy]))
+    
     # --------------------------
     # Save model results to txt
     # --------------------------
+    name_spider_plot_peak = os.path.join(
+        fig_name, "spider_scenarios_{}.pdf".format(fueltype_to_model))
+
     write_data.write_list_to_txt(
-        os.path.join(fig_name, name_spider_plot[:-3] + "txt"),
+        os.path.join(fig_name, name_spider_plot_peak[:-3] + "txt"),
+        results_txt)
+
+    name_spider_plot_trough = os.path.join(
+        fig_name, "spider_scenarios_min_summer{}.pdf".format(fueltype_to_model))
+
+    write_data.write_list_to_txt(
+        os.path.join(fig_name, name_spider_plot_trough[:-3] + "txt"),
         results_txt)
 
     # --------------------------
     # Plot figure
     # --------------------------
-    plotting_results.plot_radar_plot_multiple_lines(
-        individ_radars_to_plot_dh,
-        name_spider_plot,
+
+    # PEAK WINTER DAY PLOTS
+    plot_radar_plot_multiple_lines(
+        peak_winter_individ_radars_to_plot_dh,
+        name_spider_plot_peak,
+        plot_steps=50,
+        scenario_names=list(scenario_data.keys()),
+        plotshow=False,
+        lf_y_by=[],
+        lf_y_cy=[],
+        list_diff_max_h=results_txt)
+
+    # TROUGH Summer DAY PLOTS
+    plot_radar_plot_multiple_lines(
+        trough_summer_individ_radars_to_plot_dh,
+        name_spider_plot_trough,
         plot_steps=50,
         scenario_names=list(scenario_data.keys()),
         plotshow=False,
@@ -1033,7 +1100,7 @@ def plot_LAD_comparison_scenarios(
     # Sort regions according to size
     # -----------------
     regions = {}
-    for fueltype, fuels_regs in enumerate(scenario_data[first_scenario]['results_every_year'][2015]):
+    for fueltype, fuels_regs in enumerate(scenario_data[first_scenario]['ed_fueltype_regs_yh'][2015]):
 
         for region_array_nr, fuel_reg in enumerate(fuels_regs):
             try:
@@ -1059,7 +1126,7 @@ def plot_LAD_comparison_scenarios(
     # Plot
     # -------------------------------------
     fig = plt.figure(
-        figsize=plotting_program.cm2inch(9, 8))
+        figsize=basic_plot_functions.cm2inch(9, 8))
 
     ax = fig.add_subplot(1, 1, 1)
 
@@ -1095,12 +1162,12 @@ def plot_LAD_comparison_scenarios(
         sorted_year_data = []
         for reg_array_nr in sorted_regions_nrs:
             tot_fuel_across_fueltypes = 0
-            for fueltype, fuel_regs in enumerate(fuel_data['results_every_year'][year_to_plot]):
+            for fueltype, fuel_regs in enumerate(fuel_data['ed_fueltype_regs_yh'][year_to_plot]):
                 tot_fuel_across_fueltypes += np.sum(fuel_regs[reg_array_nr])
 
             sorted_year_data.append(tot_fuel_across_fueltypes)
 
-        tot_fuel_all_reg = np.sum(fuel_data['results_every_year'][year_to_plot])
+        tot_fuel_all_reg = np.sum(fuel_data['ed_fueltype_regs_yh'][year_to_plot])
         print("TOTAL FUEL in GWH " + str(tot_fuel_all_reg))
 
         # Calculate total annual demand
@@ -1173,3 +1240,198 @@ def plot_LAD_comparison_scenarios(
         plt.show()
     else:
         plt.close()
+
+def plot_radar_plot_multiple_lines(
+        dh_profiles,
+        fig_name,
+        plot_steps,
+        scenario_names,
+        plotshow=False,
+        lf_y_by=None,
+        lf_y_cy=None,
+        list_diff_max_h=None
+    ):
+    """Plot daily load profile on a radar plot
+
+    Arguments
+    ---------
+    dh_profile : list
+        Dh values to plot
+    fig_name : str
+        Path to save figure
+
+    SOURCE: https://python-graph-gallery.com/390-basic-radar-chart/
+    """
+    fig = plt.figure(
+        figsize=basic_plot_functions.cm2inch(9, 14))
+
+    # Get maximum demand of all lines
+    max_entry = 0
+    for line_entries in dh_profiles:
+        max_in_line = max(line_entries)
+        if max_in_line > max_entry:
+            max_entry = max_in_line
+
+    max_demand = round(max_entry, -1) + 10 # Round to nearest 10 plus add 10
+
+    nr_of_plot_steps = int(max_demand / plot_steps) + 1
+
+    axis_plots_inner = []
+    axis_plots_innter_position = []
+
+    # --------------------
+    # Ciruclar axis
+    # --------------------
+    for i in range(nr_of_plot_steps):
+        axis_plots_inner.append(plot_steps*i)
+        axis_plots_innter_position.append(str(plot_steps*i))
+
+    # Minor ticks
+    minor_tick_interval = plot_steps / 2
+    minor_ticks = []
+    for i in range(nr_of_plot_steps * 2):
+        minor_ticks.append(minor_tick_interval * i)
+
+    # ----
+    # Select color scheme
+    # ----
+
+    # Colors with scenarios
+    #color_scenarios = plotting_styles.color_list_scenarios() #Color scheme Fig 13
+
+    # Colors for plotting Fig. 13
+    #color_scenarios = plotting_styles.color_list_selection_dm() # Color scheme Fig 13
+    
+    # Color sheme multiple scenarios
+    color_scenarios = ['orange', 'darkred',  '#3b2c85', '#85cfcb'] #Lines for Fig 12
+
+    color_lines = ['black'] + color_scenarios
+    #color_lines = color_scenarios
+    years = ['2015', '2050']
+    linewidth_list = [1.0, 0.7]
+    linestyle_list = ['-', '--']
+    scenario_names.insert(0, "any")
+
+    # Iterate lines
+    cnt = -1
+    for dh_profile in dh_profiles:
+
+        if cnt >= 0:
+            cnt_year = 1
+        else:
+            cnt_year = 0
+        cnt += 1
+
+        # Line properties
+        color_line = color_lines[cnt]
+        year_line = years[cnt_year]
+
+        data = {'dh_profile': ['testname']}
+
+        # Key: Label outer circle
+        for hour in range(24):
+            data[hour] = dh_profile[hour]
+
+        # Set data
+        df = pd.DataFrame(data)
+
+        # number of variable
+        categories = list(df)[1:]
+        N = len(categories)
+
+        # We are going to plot the first line of the data frame.
+        # But we need to repeat the first value to close the circular graph:
+        values = df.loc[0].drop('dh_profile').values.flatten().tolist()
+        values += values[:1]
+
+        # What will be the angle of each axis in the plot? (we divide the plot / number of variable)
+        angles = [n / float(N) * 2 * math.pi for n in range(N)]
+        angles += angles[:1]
+
+        # Initialise the spider plot
+        ax = plt.subplot(111, polar=True)
+
+        # Change axis properties
+        ax.yaxis.grid(color='grey', linestyle=':', linewidth=0.4)
+        ax.xaxis.grid(color='lightgrey', linestyle=':', linewidth=0.4)
+
+        # Change to clockwise cirection
+        ax.set_theta_direction(-1)
+
+        # Set first hour on top
+        ax.set_theta_zero_location("N")
+
+        # Draw one axe per variable + add labels labels yet (numbers)
+        plt.xticks(
+            angles[:-1],
+            categories,
+            color='black',
+            size=8)
+
+        # Draw ylabels (numbers)
+        ax.set_rlabel_position(0)
+
+        # Working alternative
+        plt.yticks(
+            axis_plots_inner,
+            axis_plots_innter_position,
+            color="black",
+            size=8)
+
+        #ax.set_yticks(axis_plots_inner, minor=False)
+        #ax.tick_params(axis='y', pad=35) 
+        #ax.set_yticks(minor_ticks, minor=True) #Somehow not working
+
+        # Set limit to size
+        plt.ylim(0, max_demand)
+        plt.ylim(0, nr_of_plot_steps*plot_steps)
+
+        # Smooth lines
+        angles_smoothed, values_smoothed = basic_plot_functions.smooth_data(
+            angles, values, spider=True)
+
+        # Plot data
+        ax.plot(
+            angles_smoothed,
+            values_smoothed,
+            color=color_line,
+            linestyle=linestyle_list[cnt_year],
+            linewidth=linewidth_list[cnt_year],
+            label="{} {}".format(year_line, scenario_names[cnt]))
+
+        # Radar area
+        ax.fill(
+            angles_smoothed,
+            values_smoothed,
+            color_line,
+            alpha=0.05)
+
+    # ------------
+    # Write outs
+    # ------------
+    font_additional_info = plotting_styles.font_info(size=4)
+
+    for cnt, entry in enumerate(list_diff_max_h):
+        plt.text(
+            0.15,
+            0 + cnt/50,
+            entry,
+            fontsize=2,
+            fontdict=font_additional_info,
+            transform=plt.gcf().transFigure)
+
+    # ------------
+    # Legend
+    # ------------
+    plt.legend(
+        ncol=1,
+        bbox_to_anchor=(0.3, -0.05),
+        prop={'size': 4},
+        frameon=False)
+
+    plt.savefig(fig_name)
+
+    if plotshow:
+        plt.show()
+
+    plt.close()
