@@ -9,14 +9,33 @@ import numpy as np
 import geopandas as gpd
 import pandas as pd
 import palettable
-import matplotlib
-matplotlib.use('Agg') # Used to make it work in linux
 import matplotlib.pyplot as plt
 import matplotlib.patches as mpatches
+from matplotlib.patches import Circle
 from matplotlib.colors import LinearSegmentedColormap
+
 from energy_demand.basic import basic_functions
 from energy_demand.technologies import tech_related
 from energy_demand.read_write import write_data
+#matplotlib.use('Agg') # Used to make it work in linux
+
+def get_reasonable_bin_values_II(
+        data_to_plot,
+        nr_of_intervals
+    ):
+    """Get reasonable bins for 0....max value
+    """
+    def round_down(num, divisor):
+        return num - (num%divisor)
+
+    max_val = float(max(data_to_plot))
+    min_val = float(min(data_to_plot))
+
+    bin_width = max_val / nr_of_intervals
+
+    bins = [bin_width * i for i in range(nr_of_intervals)]
+
+    return bins
 
 def get_reasonable_bin_values(
         data_to_plot,
@@ -39,8 +58,8 @@ def get_reasonable_bin_values(
         # Round down and add one class for larger values
         nr_classes = round_down(classes, increments) + 1
 
-        if nr_classes > 9:
-            raise Exception("Nr of classes is too big")
+        #if nr_classes > 9:
+        #    raise Exception("Nr of classes is too big")
 
         # Classes
         min_class = round_down(min_val, increments)                              # Minimum class
@@ -50,11 +69,18 @@ def get_reasonable_bin_values(
             min_class = increments
 
         # Bin with classes
-        #logging.info("vv {}  {}".format(max_val, min_val))
-        #logging.info("pos {}  {} {}".format(min_class, max_class, increments))
         bins = list(range(int(min_class), int(max_class), int(increments)))
+
+        # ---
+        # Test that maximum 9 classes
+        # ---
+        if len(bins) > 8:
+            print("Nr of bins: " + str(len(bins)))
+            print(bins)
+            raise Exception("Too many bin classes defined" + str(len(bins)))
+
     else:
-        logging.info("Neg")
+        logging.info("Neg", flush=True)
         #lager negative values
 
         # must be of uneven length containing zero
@@ -92,11 +118,13 @@ def get_reasonable_bin_values(
 
         bins = neg_classes + [0] + pos_classes
 
-    # ---
-    # Test that maximum 9 classes
-    # ---
-    if len(bins) > 8:
-        raise Exception("Too many bin classes defined " + str(len(bins)))
+        # ---
+        # Test that maximum 9 classes
+        # ---
+        if len(bins) > 16:
+            print("Nr of bins: " + str(len(bins)))
+            print(bins)
+            raise Exception("Too many bin classes defined" + str(len(bins)))
 
     return bins
 
@@ -148,24 +176,11 @@ def user_defined_classification(
         color_list = color_list
 
     # Shorten color list
-    logging.info("BEFfd ")
-    logging.info(color_list)
-
- 
     if min(bins) > 0:
-        #bins.append(999)
-        #color_list = color_list[:len(bins)]
         color_list = color_list[:len(bins)+1]
     else:
         color_list = color_list
 
-
-    logging.info(" ")
-    logging.info("BEFORE RECLASSIFICAOTN")
-    logging.info("---")
-    logging.info(bins)
-    logging.info(color_list)
-
     # ------------------------------------------------------
     # Reclassify
     # ------------------------------------------------------
@@ -180,7 +195,8 @@ def user_defined_classification(
     # ------------------------------------------------------
     # Legend and legend labels
     # ------------------------------------------------------
-    logging.info("bins fff")
+    legend_handles = get_legend_handles(bins, color_list, color_zero, min_value, max_value)
+    '''logging.info("bins fff")
     logging.info(bins)
     logging.info(color_list)
 
@@ -238,6 +254,7 @@ def user_defined_classification(
             label=str(label_patch))
 
         legend_handles.append(patch)
+    '''
 
     plt.legend(
         handles=legend_handles,
@@ -249,6 +266,109 @@ def user_defined_classification(
 
     return reclass_lad_geopanda_shp, cmap
 
+def get_legend_handles(bins, color_list, color_zero, min_value, max_value):
+    """
+    """
+    legend_handles = []
+    small_number = 0.0001 # Small number for plotting corrrect charts
+
+    if max(bins) >= 0:
+        bins.append(999) # Append dummy last element for last class
+    else:
+        pass
+
+    for bin_nr, bin_entry in enumerate(bins):
+        if bin_nr == 0: #first bin entry
+
+            if bin_entry < 0:
+                label_patch = "> {} (min {})".format(bin_entry, min_value)
+
+                if min_value > bin_entry:
+                    #print("Classification boundry is not clever for low values")
+                    pass
+            else:
+                label_patch = "< {} (min {})".format(bin_entry, min_value)
+        elif bin_nr == len(bins)- 1: # -1 means that last bin entry
+            label_patch = "> {} (max {})".format(bins[-2], max_value)
+
+            if max_value < bin_entry:
+                #print("Classification boundry is not clever for low values")
+                pass
+        else:
+            # ----------------------------
+            # Add zero label if it exists
+            # ----------------------------
+            if bins[bin_nr - 1] == 0:
+                patch = mpatches.Patch(
+                    color=color_zero,
+                    label=str("0"))
+
+                legend_handles.append(patch)
+            else:
+                pass
+
+            # ----------------------------
+            # Other other labels
+            # ----------------------------
+            if bin_entry < 0:
+                label_patch = "{}  ―  {}".format(bins[bin_nr - 1], bin_entry - small_number)
+            else:
+                if bins[bin_nr - 1] == 0:
+                    label_patch = "{}  ―  {}".format(bins[bin_nr - 1] + small_number, bin_entry - small_number)
+                else:
+                    label_patch = "{}  ―  {}".format(bins[bin_nr - 1], bin_entry - small_number)
+
+        logging.info("------label_patch: {}  {}  {}  {}".format(label_patch, bin_entry, bins, color_list))
+
+        patch = mpatches.Patch(
+            color=color_list[bin_nr],
+            label=str(label_patch))
+
+        legend_handles.append(patch)
+
+    return legend_handles
+
+def add_simple_legend(bins, color_list, color_zero, patch_form='rectangle'):
+    """Add legend without giving the intervals
+    """
+    legend_handles = []
+
+    bin_nr = 0
+    for cnt, bin_value in enumerate(bins):
+
+        if cnt == 0: # first entry
+            if bin_value == 0:
+                label = str(bin_value)
+            else:
+                label = str("< {}".format(bins[cnt + 1]))
+        elif cnt == len(bins) - 1: #last entry
+            label = str("> {}".format(bins[cnt - 1]))
+        else:
+            label = str(bin_value)
+
+        if bin_value == 0:
+            color = color_zero
+        else:
+            color = color_list[bin_nr]
+            bin_nr += 1
+
+        if patch_form == 'rectangle':
+            patch = mpatches.Patch(
+                color=color,
+                label=label)
+        elif patch_form == 'circle':
+            patch = Circle(
+                xy=(0,0),
+                radius=1,
+                color=color,
+                label=label)
+        else:
+            raise Exception("wrong patch_form")
+
+        legend_handles.append(patch)
+
+    return legend_handles
+
 def bin_mapping(
         value_to_classify,
         nr_of_classes,
@@ -335,9 +455,6 @@ def re_classification(
     color_list_copy = copy.copy(color_list)
     bin_labels_copy = copy.copy(bin_labels)
 
-    #logging.info("KUH " + str(color_zero))
-    #logging.info(color_list_copy)
-    #logging.info(bin_labels_copy)
     # Add zero color in color list if a min_plus map
     if color_zero != False:
         insert_pos = 1
@@ -347,17 +464,11 @@ def re_classification(
         bin_labels_copy.insert(insert_pos, float(placeholder_zero_color))
     else:
         pass
-    #logging.info("KUH 2")
-    #logging.info(color_list_copy)
-    #logging.info(bin_labels_copy)
 
     # Create the custom color map
     color_bin_match_list = []
     for lbl, color in zip(bin_labels_copy, color_list_copy):
         color_bin_match_list.append((lbl, color))
-    ##logging.info("color_bin_match_list: " + str(color_bin_match_list))
-    ##logging.info(bins)
-    ##logging.info(bin_labels_copy)
 
     if 0 in bins:
         pass
@@ -371,7 +482,6 @@ def re_classification(
         color_bin_match_list)
 
     #logging.info("cmap " + str(cmap))
-
     # Reclassify
     lad_geopanda_shp['reclassified'] = lad_geopanda_shp[field_to_plot].apply(
         func=bin_mapping,
@@ -479,7 +589,6 @@ def plot_lad_national(
     # Own classification (work around)
     # -----------------------------
     if user_classification:
-        logging.info("User classification")
 
         # Color to assing zero values
         placeholder_zero_color = 0.00001
@@ -492,7 +601,6 @@ def plot_lad_national(
         # Add maximum value
         logging.info(" {} {}".format(min_value, max_value))
         logging.info("FINAL BIN before" + str(bins))
-        ###bins.append(max_value)
 
         ###logging.info("FINAL BIN " + str(bins))
         lad_geopanda_shp_reclass, cmap = user_defined_classification(
@@ -613,6 +721,7 @@ def plot_lad_national(
     # Show figure
     if plotshow:
         plt.show()
+
     plt.close()
 
 def merge_data_to_shp(shp_gdp, merge_data, unique_merge_id):
@@ -721,7 +830,7 @@ def plot_spatial_mapping_example(
 
     return
 
-def create_geopanda_files(
+def spatial_maps(
         data,
         results_container,
         path_data_results_shapefiles,
@@ -732,7 +841,7 @@ def create_geopanda_files(
         plot_crit_dict,
         base_yr
     ):
-    """Create map related files (png) from results.
+    """Create map related files from results
 
     Arguments
     ---------
@@ -747,7 +856,6 @@ def create_geopanda_files(
     """
     logging.info("... create spatial maps of results")
 
-    #base_yr = 2015
     # --------
     # Read LAD shapefile and create geopanda
     # --------
@@ -758,22 +866,27 @@ def create_geopanda_files(
     # Attribute merge unique Key
     unique_merge_id = 'name' #'geo_code'
 
+    fueltype_str_to_create_maps = [
+        'electricity'] #,
+        #'gas']
+
     # ======================================
     # Peak max h all enduses (abs)
     # ======================================
     if plot_crit_dict['plot_abs_peak_h']:
-        for year in results_container['results_every_year'].keys():
-            for fueltype in range(fueltypes_nr):
-
-                fueltype_str = tech_related.get_fueltype_str(fueltypes, fueltype)
+        for year in results_container['ed_fueltype_regs_yh'].keys():
+            
+            for fueltype_str in fueltype_str_to_create_maps:
 
+                
+                fueltype = tech_related.get_fueltype_int(fueltype_str)
+    
                 # Calculate peak h across all regions
                 field_name = 'peak_abs_h_{}_{}'.format(year, fueltype_str)
 
-
                 # Get maxium demand of 8760h for every region
-                h_max_gwh_regs = np.max(results_container['results_every_year'][year][fueltype], axis=1)
-                print("TOTAL peak fuel across all regs {} {} ".format(np.sum(h_max_gwh_regs), fueltype_str))
+                h_max_gwh_regs = np.max(results_container['ed_fueltype_regs_yh'][year][fueltype], axis=1)
+                #print("TOTAL peak fuel across all regs {} {} ".format(np.sum(h_max_gwh_regs), fueltype_str))
 
                 data_to_plot = basic_functions.array_to_dict(h_max_gwh_regs, regions)
 
@@ -815,39 +928,41 @@ def create_geopanda_files(
     # Peak max h all enduses (diff p)
     # ======================================
     if plot_crit_dict['plot_diff_peak_h']:
-        for year in results_container['results_every_year'].keys():
+        for year in results_container['ed_fueltype_regs_yh'].keys():
             if year == base_yr:
                 pass
             else:
-                for fueltype in range(fueltypes_nr):
+                
+                for fueltype_str in fueltype_str_to_create_maps:
+
                     
+                    fueltype = tech_related.get_fueltype_int(fueltype_str)
+
                     # If total sum is zero, skip
-                    if np.sum(results_container['results_every_year'][base_yr][fueltype]) == 0:
+                    if np.sum(results_container['ed_fueltype_regs_yh'][base_yr][fueltype]) == 0:
                         continue
 
-                    ##if np.isnan(np.sum(results_container['results_every_year'][base_yr][fueltype])):
+                    ##if np.isnan(np.sum(results_container['ed_fueltype_regs_yh'][base_yr][fueltype])):
                     #    logging.info("Error: Contains nan entry {} {}".format(year, fueltype))
                     #    continue
-                    #logging.info("============ {}  {}".format(fueltype, np.isnan(np.sum(results_container['results_every_year'][base_yr][fueltype]))))
-                    #logging.info(results_container['results_every_year'][base_yr][fueltype])
-                    fueltype_str = tech_related.get_fueltype_str(fueltypes, fueltype)
+                    #logging.info("============ {}  {}".format(fueltype, np.isnan(np.sum(results_container['ed_fueltype_regs_yh'][base_yr][fueltype]))))
+                    #logging.info(results_container['ed_fueltype_regs_yh'][base_yr][fueltype])
 
                     # Calculate peak h across all regions
                     field_name = 'peak_diff_p_peak_h_{}_{}'.format(year, fueltype_str)
 
                     # Get maxium demand of 8760h for every region for base year
-                    h_max_gwh_regs_by = np.max(results_container['results_every_year'][base_yr][fueltype], axis=1)
+                    h_max_gwh_regs_by = np.max(results_container['ed_fueltype_regs_yh'][base_yr][fueltype], axis=1)
 
                     # Get maxium demand of 8760h for every region for current year
-                    h_max_gwh_regs_cy = np.max(results_container['results_every_year'][year][fueltype], axis=1)
-                    print("TOTAL peak fuel across all regs {} {} ".format(np.sum(h_max_gwh_regs_cy), fueltype_str))
+                    h_max_gwh_regs_cy = np.max(results_container['ed_fueltype_regs_yh'][year][fueltype], axis=1)
+                    #print("TOTAL peak fuel across all regs {} {} ".format(np.sum(h_max_gwh_regs_cy), fueltype_str))
 
                     # Calculate difference in decimal
                     diff_p_h_max_regs = ((100 / h_max_gwh_regs_by) * h_max_gwh_regs_cy) - 100
 
                     data_to_plot = basic_functions.array_to_dict(diff_p_h_max_regs, regions)
 
-                    # Both need to be lists
                     merge_data = {
                         str(field_name): list(data_to_plot.values()),
                         str(unique_merge_id): list(regions)}
@@ -858,10 +973,12 @@ def create_geopanda_files(
                         merge_data,
                         unique_merge_id)
 
-                    bins_increments = 20 #10
+                    '''bins_increments = 25
                     bins = get_reasonable_bin_values(
                         data_to_plot=list(data_to_plot.values()),
-                        increments=bins_increments)
+                        increments=bins_increments)'''
+                    #bins = [10, 20, 40, 80, 160, 320, 640]
+                    bins = [-160, -320, -640, -80, -40, -20, -10 ,0, 10, 20, 40, 80, 160, 320, 640]
 
                     color_list, color_prop, user_classification, color_zero = colors_plus_minus_map(
                         bins=bins,
@@ -870,7 +987,6 @@ def create_geopanda_files(
                         color_zero='#ffffff',
                         color_palette='YlGnBu_7') #YlGnBu_9 #8a2be2 'PuBu_8'
    
-                    # Plot
                     plot_lad_national(
                         lad_geopanda_shp=lad_geopanda_shp,
                         legend_unit="GWh",
@@ -889,9 +1005,11 @@ def create_geopanda_files(
     # ======================================
     if plot_crit_dict['plot_load_factors']:
         for year in results_container['reg_load_factor_y'].keys():
-            for fueltype in range(fueltypes_nr):
+            
+            for fueltype_str in fueltype_str_to_create_maps:
+                
+                fueltype = tech_related.get_fueltype_int(fueltype_str)
 
-                fueltype_str = tech_related.get_fueltype_str(fueltypes, fueltype)
                 field_name = 'lf_{}_{}'.format(year, fueltype_str)
 
                 results = basic_functions.array_to_dict(
@@ -936,15 +1054,17 @@ def create_geopanda_files(
     # Load factors (difference p)
     # ======================================
     if plot_crit_dict['plot_load_factors_p']:
-        simulated_yrs = list(results_container['reg_load_factor_y'].keys())
+        sim_yrs = list(results_container['reg_load_factor_y'].keys())
 
-        final_yr = simulated_yrs[-1]
-        base_yr = simulated_yrs[0]
+        final_yr = sim_yrs[-1]
+        base_yr = sim_yrs[0]
 
-        for fueltype in range(fueltypes_nr):
-            logging.info("progress.. {}".format(fueltype))
-            fueltype_str = tech_related.get_fueltype_str(fueltypes, fueltype)
-            field_name = 'lf_diff_{}-{}_{}_'.format(base_yr, final_yr, fueltype_str)
+        
+        for fueltype_str in fueltype_str_to_create_maps:
+            fueltype = tech_related.get_fueltype_int(fueltype_str)
+            
+
+            field_name = 'lf_diff_{}-{}_{}'.format(base_yr, final_yr, fueltype_str)
 
             lf_end_yr = basic_functions.array_to_dict(
                 results_container['reg_load_factor_y'][final_yr][fueltype],
@@ -998,13 +1118,16 @@ def create_geopanda_files(
     # Population
     # ======================================
     if plot_crit_dict['plot_population']:
-        for year in results_container['results_every_year'].keys():
+        for year in results_container['ed_fueltype_regs_yh'].keys():
 
             field_name = 'pop_{}'.format(year)
 
             # Both need to be lists
+            pop_data = data['scenario_data']['population'][year]
+
             merge_data = {
-                str(field_name): data['scenario_data']['population'][year].flatten().tolist(),
+                #str(field_name): data['scenario_data']['population'][year].flatten().tolist(),
+                str(field_name): list(data['scenario_data']['population'][year]),
                 str(unique_merge_id): list(regions)}
 
             # Merge to shapefile
@@ -1030,13 +1153,12 @@ def create_geopanda_files(
     # ======================================
     # Total fuel (y) all enduses
     # ======================================
-    for year in results_container['results_every_year'].keys():
-        for fueltype in range(fueltypes_nr):
-
-            fueltype_str = tech_related.get_fueltype_str(fueltypes, fueltype)
+    for year in results_container['ed_fueltype_regs_yh'].keys():
+        
+        for fueltype_str in fueltype_str_to_create_maps:
+            fueltype = tech_related.get_fueltype_int(fueltype_str)
 
             if plot_crit_dict['plot_total_demand_fueltype']:
-                logging.info(" progress.. {}".format(fueltype))
 
                 # ---------
                 # Sum per enduse and year (y)
@@ -1045,7 +1167,7 @@ def create_geopanda_files(
 
                 # Calculate yearly sum across all regions
                 yearly_sum_gwh = np.sum(
-                    results_container['results_every_year'][year][fueltype],
+                    results_container['ed_fueltype_regs_yh'][year][fueltype],
                     axis=1)
 
                 data_to_plot = basic_functions.array_to_dict(yearly_sum_gwh, regions)
@@ -1082,11 +1204,11 @@ def create_geopanda_files(
 
                 # Calculate yearly sums
                 yearly_sum_gwh_by = np.sum(
-                    results_container['results_every_year'][base_yr][fueltype],
+                    results_container['ed_fueltype_regs_yh'][base_yr][fueltype],
                     axis=1)
 
                 yearly_sum_gwh_cy = np.sum(
-                    results_container['results_every_year'][year][fueltype],
+                    results_container['ed_fueltype_regs_yh'][year][fueltype],
                     axis=1)
 
                 # Calculate percentual difference
@@ -1117,14 +1239,12 @@ def create_geopanda_files(
                 # ----
                 # CAlculate classes for manual classification
                 # ----                
-                #logging.info("Min {}  Max {}".format(
-                #    min(list(data_to_plot.values())),
-                #     max(list(data_to_plot.values()))))
-                bins_increments = 10 #MAYBE NEEDS TO BE ADOPTED
-
+                '''bins_increments = 25 #MAYBE NEEDS TO BE ADOPTED #TODO KAMEL
                 bins = get_reasonable_bin_values(
                     data_to_plot=list(data_to_plot.values()),
-                    increments=bins_increments)
+                    increments=bins_increments)'''
+                #bins = [10, 20, 40, 80, 160, 320, 640]
+                bins = [-160, -320, -640, -80, -40, -20, -10 ,0, 10, 20, 40, 80, 160, 320, 640]
 
                 color_list, color_prop, user_classification, color_zero = colors_plus_minus_map(
                     bins=bins,
@@ -1199,12 +1319,12 @@ def colors_plus_minus_map(
         return [], color_prop, False, False
 
     elif min(bins) < 0:
-        logging.info("negative bins" + str(bins))
-
-        if len(bins) > 10:
-            raise Exception("Too many bins defined: Change interval criteria")
-        if len(bins) == 10 or len(bins) == 9: # add extra color to reach 10 colors
-
+        #logging.info("negative bins" + str(bins))
+        #print("negative bins" + str(bins))
+        #if len(bins) > 10:
+        #    raise Exception("Too many bins defined: Change interval criteria")
+        #if len(bins) == 10 or len(bins) == 9: # add extra color to reach 10 colors
+        if 1== 1:
             if color_order:
                 color_list_pos = getattr(palettable.colorbrewer.sequential, 'Reds_9').hex_colors
                 color_list_pos.append('#330808')
@@ -1261,7 +1381,7 @@ def colors_plus_minus_map(
             nr_of_cat_pos = int(len(bins))
             for i in range(nr_of_cat_pos + 1): #add one to get class up to zero
                 color_list.append(color_list_neg[i])
-            
+
             color_list.insert(0, color_zero) # Add 0 color
         else:
             nr_of_cat_neg = 0
@@ -1274,7 +1394,7 @@ def colors_plus_minus_map(
                     nr_of_cat_pos += 1
                 else:
                     pass
-    
+
             for i in range(nr_of_cat_neg + 1): #add one to get class before first bin
                 color_list.append(color_list_neg[i])
 
@@ -1284,7 +1404,7 @@ def colors_plus_minus_map(
         return color_list, 'user_defined', True, color_zero
 
     elif min(bins) > 0:
-        print("positive bins " + str(bins))
+
         color_list = []
 
         if len(bins) == 10 or len(bins) == 9: # add extra color to reach 10 colors
