@@ -12,11 +12,23 @@ import fiona
 import pandas as pd
 from shapely.geometry import shape, mapping
 import numpy as np
+from ruamel.yaml import YAML
 
 from energy_demand.technologies import tech_related
 from energy_demand.profiles import load_profile
 from energy_demand.basic import lookup_tables
 
+def read_yaml(file_path):
+    """Parse yaml config file into plain data (lists, dicts and simple values)
+
+    Parameters
+    ----------
+    file_path : str
+        The path of the configuration file to parse
+    """
+    with open(file_path, 'r') as file_handle:
+        return YAML(typ='unsafe').load(file_handle)
+
 class TechnologyData(object):
     """Class to store technology related data
 
@@ -241,13 +253,17 @@ class ServiceSwitch(object):
         """
         setattr(self, name, value)
 
-def read_in_results(path_runs, seasons, model_yeardays_daytype):
+def read_in_results(
+        path_result,
+        seasons,
+        model_yeardays_daytype
+    ):
     """Read and post calculate results from txt files
     and store into container
 
     Arguments
     ---------
-    path_runs : str
+    path_result : str
         Paths
     seasons : dict
         seasons
@@ -260,62 +276,90 @@ def read_in_results(path_runs, seasons, model_yeardays_daytype):
 
     results_container = {}
 
-    # -------------
-    # Fuels
-    # -------------
-    results_container['results_enduse_every_year'] = read_enduse_specific_results(
-        path_runs)
+    # -----------------
+    # Read in demands
+    # -----------------
+    try:
+        results_container['results_enduse_every_year'] = read_enduse_specific_results(
+            path_result)
+    except:
+        pass
+    try:
+        print("path_result " + str(path_result))
+        results_container['ed_fueltype_regs_yh'] = read_results_yh(
+            path_result, 'ed_fueltype_regs_yh')
+    except:
+        pass
+    # Read in residential demands
+    try:
+        results_container['residential_results'] = read_results_yh(
+            path_result, 'residential_results')
+    except:
+        pass
 
-    results_container['results_every_year'] = read_results_yh(path_runs)
+    # Calculate total demand per fueltype for every hour
+    try:
+        tot_fueltype_yh = {}
+        for year in results_container['ed_fueltype_regs_yh']:
+            nr_of_fueltypes = results_container['ed_fueltype_regs_yh'][year].shape[0]
+            tot_fueltype_yh[year] = np.zeros((nr_of_fueltypes, 8760))
+
+        for year, ed_regs_yh in results_container['ed_fueltype_regs_yh'].items():
+            fuel_yh = np.sum(ed_regs_yh, axis=1) #Sum across all regions
+            tot_fueltype_yh[year] += fuel_yh
+
+        results_container['tot_fueltype_yh'] = tot_fueltype_yh
+    except:
+        pass
 
     # -----------------
     # Peak calculations
     # -----------------
-    results_container['ed_peak_h'] = {}
-    results_container['ed_peak_regs_h'] = {}
+    try:
+        results_container['ed_peak_h'] = {}
+        results_container['ed_peak_regs_h'] = {}
 
-    for year, year_yh in results_container['results_every_year'].items():
+        for year, ed_fueltype_reg_yh in results_container['ed_fueltype_regs_yh'].items():
+            results_container['ed_peak_h'][year] = {}
+            results_container['ed_peak_regs_h'][year] = {}
 
-        results_container['ed_peak_h'][year] = {}
-        results_container['ed_peak_regs_h'][year] = {}
+            for fueltype_int, ed_reg_yh in enumerate(ed_fueltype_reg_yh):
+                fueltype_str = tech_related.get_fueltype_str(lookups['fueltypes'], fueltype_int)
 
-        for fueltype_int, reg_ed_yh in enumerate(year_yh):
+                # Calculate peak per fueltype for all regions (ed_reg_yh= np.array(fueltype, reg, yh))
+                all_regs_yh = np.sum(ed_reg_yh, axis=0)    # sum regs
+                peak_h = np.max(all_regs_yh)               # select max of 8760 h
+                results_container['ed_peak_h'][year][fueltype_str] = peak_h
+                results_container['ed_peak_regs_h'][year][fueltype_str] = np.max(ed_reg_yh, axis=1)
 
-            fueltype_str = tech_related.get_fueltype_str(lookups['fueltypes'], fueltype_int)
+        # -------------
+        # Load factors
+        # -------------
+        results_container['reg_load_factor_y'] = read_lf_y(
+            os.path.join(path_result, "result_reg_load_factor_y"))
+        results_container['reg_load_factor_yd'] = read_lf_y(
+            os.path.join(path_result, "result_reg_load_factor_yd"))
 
-            # Calculate peak per fueltype for all regions (reg_ed_yh= np.array(fueltype, reg, yh))
-            all_regs_yh = np.sum(reg_ed_yh, axis=0)    # sum regs
-            peak_h = np.max(all_regs_yh)               # select max of 8760 h
-            results_container['ed_peak_h'][year][fueltype_str] = peak_h
-            results_container['ed_peak_regs_h'][year][fueltype_str] = np.max(reg_ed_yh, axis=1)
-
-    # -------------
-    # Load factors
-    # -------------
-    results_container['reg_load_factor_y'] = read_lf_y(
-        os.path.join(path_runs, "result_reg_load_factor_y"))
-    results_container['reg_load_factor_yd'] = read_lf_y(
-        os.path.join(path_runs, "result_reg_load_factor_yd"))
-
-    results_container['load_factor_seasons'] = {}
-    results_container['load_factor_seasons']['winter'] = read_lf_y(
-        os.path.join(path_runs, "result_reg_load_factor_winter"))
-    results_container['load_factor_seasons']['spring'] = read_lf_y(
-        os.path.join(path_runs, "result_reg_load_factor_spring"))
-    results_container['load_factor_seasons']['summer'] = read_lf_y(
-        os.path.join(path_runs, "result_reg_load_factor_summer"))
-    results_container['load_factor_seasons']['autumn'] = read_lf_y(
-        os.path.join(path_runs, "result_reg_load_factor_autumn"))
-
-    # -------------
-    # Post-calculations
-    # -------------
-    # Calculate average per season and fueltype for every fueltype
-    results_container['av_season_daytype_cy'], results_container['season_daytype_cy'] = calc_av_per_season_fueltype(
-        results_container['results_every_year'],
-        seasons,
-        model_yeardays_daytype)
+        # -------------
+        # Post-calculations
+        # -------------
+        # Calculate average per season and fueltype for every fueltype
+        results_container['av_season_daytype_cy'], results_container['season_daytype_cy'] = calc_av_per_season_fueltype(
+            results_container['ed_fueltype_regs_yh'],
+            seasons,
+            model_yeardays_daytype)
 
+        '''results_container['load_factor_seasons'] = {}
+        results_container['load_factor_seasons']['winter'] = read_lf_y(
+            os.path.join(path_result, "result_reg_load_factor_winter"))
+        results_container['load_factor_seasons']['spring'] = read_lf_y(
+            os.path.join(path_result, "result_reg_load_factor_spring"))
+        results_container['load_factor_seasons']['summer'] = read_lf_y(
+            os.path.join(path_result, "result_reg_load_factor_summer"))
+        results_container['load_factor_seasons']['autumn'] = read_lf_y(
+            os.path.join(path_result, "result_reg_load_factor_autumn"))'''
+    except:
+        pass
     logging.info("... Reading in results finished")
     return results_container
 
@@ -360,7 +404,7 @@ def calc_av_per_season_fueltype(results_every_year, seasons, model_yeardays_dayt
 
     return dict(av_season_daytype_cy), dict(season_daytype_cy)
 
-def read_results_yh(path_to_folder):
+def read_results_yh(path_to_folder, name_of_folder):
     """Read results
 
     Arguments
@@ -379,16 +423,15 @@ def read_results_yh(path_to_folder):
     """
     results = {}
 
-    path_to_folder = os.path.join(path_to_folder, 'result_tot_yh')
+    path_to_folder = os.path.join(path_to_folder, name_of_folder)
 
     all_txt_files_in_folder = os.listdir(path_to_folder)
 
-    # Iterate files in folder
     for file_path in all_txt_files_in_folder:
         try:
             path_file_to_read = os.path.join(path_to_folder, file_path)
             file_path_split = file_path.split("__")
-            year = int(file_path_split[1])
+            year = int(file_path_split[1][:-4])
 
             results[year] = np.load(path_file_to_read)
 
@@ -436,8 +479,6 @@ def read_enduse_specific_results(path_to_folder):
     all_txt_files_in_folder = os.listdir(path_results)
 
     for file_path in all_txt_files_in_folder:
-
-        
         path_file_to_read = os.path.join(path_results, file_path)
         file_path_split = file_path.split("__")
 
@@ -469,6 +510,10 @@ def read_fuel_ss(path_to_csv, fueltypes_nr):
         Service sectors
     enduses : list
         Service enduses
+    
+    Info of categories
+    ------------------
+    https://assets.publishing.service.gov.uk/government/uploads/system/uploads/attachment_data/file/565748/BEES_overarching_report_FINAL.pdf
     """
     lookups = lookup_tables.basic_lookups()
     fueltypes_lu = lookups['fueltypes']
@@ -536,7 +581,7 @@ def read_load_shapes_tech(path_to_csv):
 
     return load_shapes_dh
 
-def service_switch(path_to_csv, technologies, base_yr=2015):
+def service_switch(df_service_switches):
     """This function reads in service assumptions from csv file,
     tests whether the maximum defined switch is larger than
     possible for a technology.
@@ -572,33 +617,28 @@ def service_switch(path_to_csv, technologies, base_yr=2015):
         sector                      [str]   Optional sector specific info where switch applies
     """
     service_switches = []
+    default_parameter = 999.0   #default parameter
 
-    if os.path.isfile(path_to_csv):
+    for i in df_service_switches.index:
+        enduse = df_service_switches.at[i, 'enduses_service_switch']
+        tech = df_service_switches.at[i, 'tech']
+        service_share_ey = df_service_switches.at[i, 'switches_service']
+        switch_yr = df_service_switches.at[i, 'end_yr']
+        sector = df_service_switches.at[i, 'sector']
 
-        # Read switches
-        raw_csv_file = pd.read_csv(path_to_csv)
-
-        for _index, row in raw_csv_file.iterrows():
+        if sector == 'None':
+            sector = None
 
+        if float(service_share_ey) == default_parameter:
+            pass
+        else:
             service_switches.append(
                 ServiceSwitch(
-                    enduse=str(row['enduse']),
-                    technology_install=str(row['tech']),
-                    service_share_ey=float(row['service_share_ey']),
-                    switch_yr=float(row['switch_yr']),
-                    sector=row['sector']))
-
-        # Test if more service is provided as input than possible to maximum switch
-        for entry in service_switches:
-            if entry.service_share_ey > technologies[entry.technology_install].tech_max_share:
-                raise Exception(
-                    "Input error: more service provided for tech '{}' in enduse '{}' than max possible".format(
-                        entry.enduse, entry.technology_install))
-
-            if entry.switch_yr <= base_yr:
-                raise Exception("Input error service switch: switch_yr must be in the future")
-    else:
-        pass
+                    enduse=str(enduse),
+                    technology_install=str(tech),
+                    service_share_ey=float(service_share_ey),
+                    switch_yr=float(switch_yr),
+                    sector=sector))
 
     return service_switches
 
@@ -646,7 +686,7 @@ def read_fuel_switches(
     fuel_switches = []
 
     if os.path.isfile(path_to_csv):
-        # Read switches
+
         raw_csv_file = pd.read_csv(path_to_csv)
 
         for index, row in raw_csv_file.iterrows():
@@ -746,7 +786,6 @@ def read_technologies(path_to_csv):
     dict_technologies = {}
     dict_tech_lists = {}
 
-    # Read csv
     raw_csv_file = pd.read_csv(path_to_csv)
 
     for index, row in raw_csv_file.iterrows():
@@ -797,6 +836,9 @@ def read_fuel_rs(path_to_csv):
     the first row is the fuel_ID
     The header is the sub_key
     """
+    dummy_sector = None
+    sectors = [dummy_sector]
+
     fuels = {}
 
     # Read csv
@@ -813,9 +855,8 @@ def read_fuel_rs(path_to_csv):
 
     # Iterate columns and convert to array
     for enduse in raw_csv_file.columns[1:]: # skip for column
-        fuels[enduse] = raw_csv_file[enduse].values
-
-    sectors = [None]
+        fuels[enduse] = {}
+        fuels[enduse][dummy_sector] = raw_csv_file[enduse].values
 
     return fuels, sectors, list(enduses)
 
@@ -1026,7 +1067,6 @@ def read_scenaric_population_data(result_path):
 
     all_txt_files_in_folder = os.listdir(result_path)
 
-    # Iterate files
     for file_path in all_txt_files_in_folder:
         path_file_to_read = os.path.join(result_path, file_path)
         file_path_split = file_path.split("__")
@@ -1231,9 +1271,7 @@ def get_position(headings, name):
     position : int
         Position in list
     """
-    for position, value in enumerate(headings):
-        if str(value) == str(name):
-            return position
+    return headings.index(name)
 
 def read_np_array_from_txt(path_file_to_read):
     """Read np array from textfile
@@ -1303,4 +1341,3 @@ def load_full_paramter_values(file_path):
     gp_file = pd.read_csv(file_path)
 
     return gp_file
-        
\ No newline at end of file
