@@ -3,13 +3,16 @@
 import logging
 import copy
 from collections import defaultdict
+import numpy as np
+
 from energy_demand.profiles import hdd_cdd
 from energy_demand.basic import testing_functions
 from energy_demand.basic import lookup_tables
 from energy_demand.read_write import data_loader
 from energy_demand.scripts import init_scripts
+from energy_demand.technologies import tech_related
 
-def disaggregate_demand(data):
+def disaggr_demand(data, crit_temp_min_max, spatial_calibration=False):
     """Disaggregated demand
 
     Arguments
@@ -21,33 +24,133 @@ def disaggregate_demand(data):
     --------
     disagg : dict
         Disaggregated energy demand
+    spatial_calibration : bool
+        Criteria wheter base year data should be used to
+        calibrate spatial disaggregation (non-residential demand)
     """
     disagg = {}
 
     # ===========================================
     # I. Disaggregation
     # ===========================================
+    base_yr = data['assumptions'].base_yr
 
     # Load data for disaggregateion
     data['scenario_data']['employment_stats'] = data_loader.read_employment_stats(
         data['paths']['path_employment_statistics'])
 
-    # Load population data for disaggregation
-    pop_for_disagg = data['pop_for_disag']
-
     # Disaggregate fuel for all regions
     disagg['residential'], disagg['service'], disagg['industry'] = disaggregate_base_demand(
-        pop_for_disagg,
+        data['pop_for_disag'],
         data['regions'],
         data['fuels'],
         data['scenario_data'],
         data['assumptions'],
         data['reg_coord'],
-        data['weather_stations'],
-        data['temp_data'],
+        data['weather_stations'], # Base year data used to disaggregate demand
+        data['temp_data'][base_yr], # Base year data used to disaggregate demand
         data['sectors'],
         data['enduses'],
-        data['service_building_count'])
+        data['service_building_count'],
+        crit_temp_min_max)
+
+    if spatial_calibration:
+        '''The spatial disaggregation of non-residential demand
+        can be calibrated for gas and electricity based on actual
+        measured demand data
+
+        Note: All other fueltypes are not scaled
+        '''
+        calibrate_residential = False # Calibrate residential demands
+        calibrate_non_residential = True # Calibrate non residential demands
+
+        # Non-residential electricity regional demands of base year for electrictiy and gas
+        fueltype_elec = tech_related.get_fueltype_int('electricity')
+        fueltype_gas = tech_related.get_fueltype_int('gas')
+
+        if calibrate_non_residential:
+
+            valid_non_resid_elec = data_loader.read_lad_demands(
+                data['paths']['val_subnational_elec_non_residential'])
+
+            valid_non_resid_gas = data_loader.read_lad_demands(
+                data['paths']['val_subnational_gas_non_residential'])
+
+            # Calculate and apply regional calibration factor
+            for region in data['regions']:
+
+                # Total modeleld non_residential
+                service_demand_elec = 0
+                service_demand_gas = 0
+                for enduse_data in disagg['service'][region].values():
+                    for sector_data in enduse_data.values():
+                        service_demand_elec += np.sum(sector_data[fueltype_elec])
+                        service_demand_gas += np.sum(sector_data[fueltype_gas])
+
+                industry_demand_elec = 0
+                industry_demand_gas = 0
+                for enduse_data in disagg['industry'][region].values():
+                    for sector_data in enduse_data.values():
+                        industry_demand_elec += np.sum(sector_data[fueltype_elec])
+                        industry_demand_gas += np.sum(sector_data[fueltype_gas])
+
+                modelled_elec = service_demand_elec + industry_demand_elec
+                modelled_gas = service_demand_gas + industry_demand_gas
+
+                # Calculate calibration factor
+                try:
+                    f_spatial_calibration_elec = valid_non_resid_elec[region] / modelled_elec
+                except KeyError: # No real data available
+                    f_spatial_calibration_elec = 1
+
+                try:
+                    f_spatial_calibration_gas = valid_non_resid_gas[region] / modelled_gas
+                except KeyError: # No real data available
+                    f_spatial_calibration_gas = 1
+
+                # Apply calibration factor to spatial disaggregation
+                for enduse in disagg['service'][region].keys():
+                    for sector in disagg['service'][region][enduse]:
+                        disagg['service'][region][enduse][sector][fueltype_elec] *= f_spatial_calibration_elec
+                        disagg['service'][region][enduse][sector][fueltype_gas] *= f_spatial_calibration_gas
+                for enduse in disagg['industry'][region].keys():
+                    for sector in disagg['industry'][region][enduse]:
+                        disagg['industry'][region][enduse][sector][fueltype_elec] *= f_spatial_calibration_elec
+                        disagg['industry'][region][enduse][sector][fueltype_gas] *= f_spatial_calibration_gas
+
+        if calibrate_residential:
+            valid_resid_elec = data_loader.read_lad_demands(
+                data['paths']['val_subnational_elec_residential'])
+            valid_resid_gas = data_loader.read_lad_demands(
+                data['paths']['val_subnational_gas_residential'])
+
+            # Calculate and apply regional calibration factor
+            for region in data['regions']:
+
+                # Total modeleld residential
+                modelled_elec = 0
+                modelled_gas = 0
+                for enduse_data in disagg['residential'][region].values():
+                    modelled_elec += np.sum(enduse_data[fueltype_elec])
+                    modelled_gas += np.sum(enduse_data[fueltype_gas])
+
+                # Calculate calibration factor
+                try:
+                    real_elec = valid_resid_elec[region]
+                    f_spatial_calibration_elec = real_elec / modelled_elec
+                    real_gas = valid_resid_gas[region]
+                    f_spatial_calibration_gas = real_gas / modelled_gas
+                except KeyError:
+                    # No real data available
+                    f_spatial_calibration_elec = 1
+                    f_spatial_calibration_gas = 1
+
+                # Apply calibration factor to spatial disaggregation
+                for enduse in disagg['residential'][region].keys():
+                    disagg['residential'][region][enduse][fueltype_elec] *= f_spatial_calibration_elec
+                    disagg['residential'][region][enduse][fueltype_gas] *= f_spatial_calibration_gas
+    else:
+        pass
 
     # Sum demand across all sectors for every region
     disagg['ss_fuel_disagg_sum_all_sectors'] = init_scripts.sum_across_sectors_all_regs(
@@ -58,19 +161,14 @@ def disaggregate_demand(data):
 
     # Sum demand across all submodels and sectors for every region
     disagg['tot_disaggregated_regs'] = init_scripts.sum_across_all_submodels_regs(
-        data['lookups']['fueltypes_nr'],
         data['regions'],
-        [disagg['residential'],
-        disagg['service'],
-        disagg['industry']])
+        [disagg['residential'], disagg['service'], disagg['industry']])
 
     disagg['tot_disaggregated_regs_residenital'] = init_scripts.sum_across_all_submodels_regs(
-        data['lookups']['fueltypes_nr'],
         data['regions'],
         [disagg['residential']])
 
     disagg['tot_disaggregated_regs_non_residential'] = init_scripts.sum_across_all_submodels_regs(
-        data['lookups']['fueltypes_nr'],
         data['regions'],
         [disagg['service'], disagg['industry']])
 
@@ -100,7 +198,8 @@ def disaggregate_base_demand(
         temp_data,
         sectors,
         enduses,
-        service_building_count
+        service_building_count,
+        crit_temp_min_max
     ):
     """This function disaggregates fuel demand based on
     region specific parameters for the base year. The residential,
@@ -156,7 +255,8 @@ def disaggregate_base_demand(
         enduses['residential'],
         crit_limited_disagg_pop_hdd,
         crit_limited_disagg_pop,
-        crit_full_disagg)
+        crit_full_disagg,
+        crit_temp_min_max)
 
     # Service
     ss_fuel_disagg = ss_disaggregate(
@@ -173,7 +273,8 @@ def disaggregate_base_demand(
         sectors['service'],
         crit_limited_disagg_pop_hdd,
         crit_limited_disagg_pop,
-        crit_full_disagg)
+        crit_full_disagg,
+        crit_temp_min_max)
 
     # Industry
     is_fuel_disagg = is_disaggregate(
@@ -187,9 +288,9 @@ def disaggregate_base_demand(
         sectors['industry'],
         scenario_data['employment_stats'],
         pop_for_disagg,
-        census_disagg=census_disagg)
+        census_disagg=census_disagg,
+        crit_temp_min_max=crit_temp_min_max)
 
-    logging.info("Finished disaggregation")
     return dict(rs_fuel_disagg), dict(ss_fuel_disagg), dict(is_fuel_disagg)
 
 def ss_disaggregate(
@@ -206,7 +307,8 @@ def ss_disaggregate(
         sectors,
         crit_limited_disagg_pop_hdd,
         crit_limited_disagg_pop,
-        crit_full_disagg
+        crit_full_disagg,
+        crit_temp_min_max=False
     ):
     """Disaggregate fuel for service submodel (per enduse and sector)
 
@@ -222,18 +324,20 @@ def ss_disaggregate(
     # Calculate heating degree days for regions
     # ---------------------------------------
     ss_hdd_individ_region = hdd_cdd.get_hdd_country(
-        t_base_heating=assumptions.t_bases.ss_t_heating_by,
+        t_base_heating=assumptions.t_bases.ss_t_heating,
         regions=regions,
         temp_data=temp_data,
         reg_coord=reg_coord,
-        weather_stations=weather_stations)
+        weather_stations=weather_stations,
+        crit_temp_min_max=crit_temp_min_max)
 
     ss_cdd_individ_region = hdd_cdd.get_cdd_country(
-        t_base_cooling=assumptions.t_bases.ss_t_cooling_by,
+        t_base_cooling=assumptions.t_bases.ss_t_cooling,
         regions=regions,
         temp_data=temp_data,
         reg_coord=reg_coord,
-        weather_stations=weather_stations)
+        weather_stations=weather_stations,
+        crit_temp_min_max=crit_temp_min_max)
 
     # ---------------------------------------------
     # Get all regions with floor area data
@@ -315,14 +419,6 @@ def ss_disaggr(
     """Disaggregating
     """
     ss_fuel_disagg = {}
-
-    # Total floor area for every enduse per sector
-    '''national_floorarea_by_sector = {}
-    for sector in sectors:
-        national_floorarea_by_sector[sector] = 0
-        for region in regions:
-            national_floorarea_by_sector[sector] += scenario_data['floor_area']['ss_floorarea'][base_yr][region][sector]'''
-
     tot_pop = 0
     tot_cdd = 0
     tot_hdd = 0
@@ -349,7 +445,6 @@ def ss_disaggr(
     for region in all_regions:
         reg_hdd = ss_hdd_individ_region[region]
         reg_cdd = ss_cdd_individ_region[region]
-        #reg_pop = scenario_data['population'][base_yr][region]
         reg_pop = pop_for_disagg[base_yr][region]
         tot_pop += reg_pop
         tot_cdd += reg_cdd
@@ -385,7 +480,6 @@ def ss_disaggr(
         # Regional factors
         reg_hdd = ss_hdd_individ_region[region]
         reg_cdd = ss_cdd_individ_region[region]
-        #reg_pop = scenario_data['population'][base_yr][region]
         reg_pop = pop_for_disagg[base_yr][region]
         p_pop = reg_pop / tot_pop
 
@@ -436,6 +530,12 @@ def ss_disaggr(
 
                         reg_diasg_factor = p_reg_sector_building_cnt
 
+                # If reg_diasg_factor, replace by population disaggregation
+                if np.isnan(reg_diasg_factor):
+                    reg_diasg_factor = p_pop
+                else:
+                    pass
+
                 ss_fuel_disagg[region][enduse][sector] = ss_national_fuel[enduse][sector] * reg_diasg_factor
 
     return dict(ss_fuel_disagg)
@@ -451,7 +551,8 @@ def is_disaggregate(
         sectors,
         employment_statistics,
         pop_for_disagg,
-        census_disagg
+        census_disagg,
+        crit_temp_min_max=False
     ):
     """Disaggregate industry related fuel for sector and enduses with
     employment statistics
@@ -479,12 +580,12 @@ def is_disaggregate(
         reg, enduse, sector
     """
     is_hdd_individ_region = hdd_cdd.get_hdd_country(
-        t_base_heating=assumptions.t_bases.is_t_heating_by,
+        t_base_heating=assumptions.t_bases.is_t_heating,
         regions=regions,
         temp_data=temp_data,
         reg_coord=reg_coord,
-        weather_stations=weather_stations)
-    logging.debug("... disaggregate industry demand")
+        weather_stations=weather_stations,
+        crit_temp_min_max=crit_temp_min_max)
 
     is_fuel_disagg = {}
 
@@ -492,7 +593,7 @@ def is_disaggregate(
     tot_pop = 0
     tot_pop_hdd = 0
     for region in regions:
-        tot_pop_hdd +=pop_for_disagg[assumptions.base_yr][region] * is_hdd_individ_region[region]
+        tot_pop_hdd += pop_for_disagg[assumptions.base_yr][region] * is_hdd_individ_region[region]
         tot_pop += pop_for_disagg[assumptions.base_yr][region]
 
     if not census_disagg:
@@ -585,11 +686,8 @@ def is_disaggregate(
         # --------------------------------------------------
         for region in regions:
             is_fuel_disagg[region] = {}
-
-            #reg_pop = scenario_data['population'][assumptions.base_yr][region]
             reg_pop = pop_for_disagg[assumptions.base_yr][region]
 
-            # Iterate sector
             for enduse in enduses:
                 is_fuel_disagg[region][enduse] = {}
 
@@ -629,6 +727,12 @@ def is_disaggregate(
                             except ZeroDivisionError:
                                 reg_disagg_f = 0 #No employment for this sector for this region
 
+                        # If nan, use population
+                        if np.isnan(reg_disagg_f):
+                            reg_disagg_f = reg_pop / tot_pop
+                        else:
+                            pass
+
                         # Disaggregated national fuel
                         is_fuel_disagg[region][enduse][sector] = is_national_fuel[enduse][sector] * reg_disagg_f
 
@@ -653,7 +757,9 @@ def rs_disaggregate(
         enduses,
         crit_limited_disagg_pop_hdd,
         crit_limited_disagg_pop,
-        crit_full_disagg
+        crit_full_disagg,
+        crit_temp_min_max=False,
+        dummy_sector=None
     ):
     """Disaggregate residential fuel demand
 
@@ -675,17 +781,18 @@ def rs_disaggregate(
     to enduse (see Documentation)
     """
     logging.debug("... disagreggate residential demand")
-    rs_fuel_disagg = {}
 
+    rs_fuel_disagg = {}
     # ---------------------------------------
     # Calculate heating degree days for regions
     # ---------------------------------------
     rs_hdd_individ_region = hdd_cdd.get_hdd_country(
-        t_base_heating=assumptions.t_bases.rs_t_heating_by,
+        t_base_heating=assumptions.t_bases.rs_t_heating,
         regions=regions,
         temp_data=temp_data,
         reg_coord=reg_coord,
-        weather_stations=weather_stations)
+        weather_stations=weather_stations,
+        crit_temp_min_max=crit_temp_min_max)
 
     # ---------------------------------------
     # Get all regions with  floor area data
@@ -698,7 +805,7 @@ def rs_disaggregate(
     # Disaggregate for region without floor
     # area by population and hdd (set crit_full_disagg as False)
     # ====================================
-    rs_fuel_disagg_only_pop = rs_disaggr(
+    rs_fuel_regions = rs_disaggr(
         all_regions=regions,
         regions=assumptions.rs_regions_without_floorarea,
         base_yr=assumptions.base_yr,
@@ -708,14 +815,15 @@ def rs_disaggregate(
         rs_national_fuel=rs_national_fuel,
         crit_limited_disagg_pop=False, #True, #False,
         crit_limited_disagg_pop_hdd=True, #True, #Set to true
-        crit_full_disagg=False)
-    rs_fuel_disagg.update(rs_fuel_disagg_only_pop)
+        crit_full_disagg=False,
+        dummy_sector=dummy_sector)
+    rs_fuel_disagg.update(rs_fuel_regions)
 
     # Substract fuel for regions where only population was used for disaggregation from total
     rs_national_fuel_remaining = copy.deepcopy(rs_national_fuel)
     for enduse in rs_national_fuel:
         for reg in rs_fuel_disagg:
-            rs_national_fuel_remaining[enduse] -= rs_fuel_disagg[reg][enduse]
+            rs_national_fuel_remaining[enduse][dummy_sector] -= rs_fuel_disagg[reg][enduse][dummy_sector]
 
     # ====================================
     # Disaggregate for region with floor area
@@ -730,14 +838,13 @@ def rs_disaggregate(
         rs_national_fuel=rs_national_fuel_remaining,
         crit_limited_disagg_pop=crit_limited_disagg_pop,
         crit_limited_disagg_pop_hdd=crit_limited_disagg_pop_hdd,
-        crit_full_disagg=crit_full_disagg)
+        crit_full_disagg=crit_full_disagg,
+        dummy_sector=dummy_sector)
     rs_fuel_disagg.update(rs_fuel_disagg_full_data)
 
-    # -----------------
     # Check if total fuel is the same before and after aggregation
-    #------------------ 
     testing_functions.control_disaggregation(
-        rs_fuel_disagg, rs_national_fuel, enduses)
+        rs_fuel_disagg, rs_national_fuel, enduses, [dummy_sector])
 
     return rs_fuel_disagg
 
@@ -751,11 +858,12 @@ def rs_disaggr(
         rs_national_fuel,
         crit_limited_disagg_pop,
         crit_limited_disagg_pop_hdd,
-        crit_full_disagg
+        crit_full_disagg,
+        dummy_sector
     ):
     """Disaggregate residential enduses
     """
-    fuel_disagg = defaultdict(dict)
+    fuel_disagg = {}
 
     total_pop = 0
     total_hdd_floorarea = 0
@@ -766,7 +874,6 @@ def rs_disaggr(
     for region in all_regions:
         reg_hdd = rs_hdd_individ_region[region]
         reg_floor_area = scenario_data['floor_area']['rs_floorarea'][base_yr][region]
-        #reg_pop = scenario_data['population'][base_yr][region]
         reg_pop = pop_for_disagg[base_yr][region]
 
         # National dissagregation factors
@@ -780,10 +887,10 @@ def rs_disaggr(
     # Disaggregate according to enduse
     # ---------------------------------------
     for region in regions:
+        fuel_disagg[region] = {}
         reg_hdd = rs_hdd_individ_region[region]
         reg_floor_area = scenario_data['floor_area']['rs_floorarea'][base_yr][region]
         reg_hdd_floor_area = reg_hdd * reg_floor_area
-        #reg_pop = scenario_data['population'][base_yr][region]
         reg_pop = pop_for_disagg[base_yr][region]
         reg_pop_hdd = reg_pop * reg_hdd
 
@@ -792,6 +899,7 @@ def rs_disaggr(
 
         # Disaggregate fuel depending on end_use
         for enduse in rs_national_fuel:
+            fuel_disagg[region][enduse] = {}
             if crit_limited_disagg_pop and not crit_limited_disagg_pop_hdd and not crit_full_disagg:
 
                 # ----------------------------------
@@ -821,8 +929,13 @@ def rs_disaggr(
                 else:
                     reg_diasg_factor = p_pop
 
+            if np.isnan(reg_diasg_factor):
+                reg_diasg_factor = p_pop
+            else:
+                pass
+
             # Disaggregate
-            fuel_disagg[region][enduse] = rs_national_fuel[enduse] * reg_diasg_factor
+            fuel_disagg[region][enduse][dummy_sector] = rs_national_fuel[enduse][dummy_sector] * reg_diasg_factor
 
     return dict(fuel_disagg)
 
