@@ -81,7 +81,6 @@ def sum_fuel_across_sectors(fuels):
 
 def get_share_s_tech_ey(
         service_switches,
-        specified_tech_enduse_by
     ):
     """Get fraction of service for each technology
     defined in a switch for every narrative year
@@ -90,8 +89,6 @@ def get_share_s_tech_ey(
     ---------
     service_switches : list
         Service switches
-    specified_tech_enduse_by : list
-        Technologies defined per enduse for base year
 
     Return
     ------
@@ -119,7 +116,7 @@ def create_switches_from_s_shares(
         enduse,
         s_tech_by_p,
         switch_technologies,
-        specified_tech_enduse_by,
+        tech_enduse_by,
         enduse_switches,
         s_tot_defined,
         sector,
@@ -139,8 +136,8 @@ def create_switches_from_s_shares(
         Service share per technology in base year
     switch_technologies : list
         Technologies involved in switch
-    specified_tech_enduse_by : list
-        Technologies per enduse
+    tech_enduse_by : list
+        Technologies of enduse and sector
     enduse_switches : list
         Switches
     s_tot_defined : float
@@ -160,57 +157,68 @@ def create_switches_from_s_shares(
     # Calculate relative by proportion of not assigned technologies in base year
     tech_not_assigned_by_p = {}
 
-    for tech in specified_tech_enduse_by[enduse]:
+    for tech in tech_enduse_by:
         if tech not in switch_technologies:
             tech_not_assigned_by_p[tech] = s_tech_by_p[sector][enduse][tech]
 
     # Normalise: convert to percentage
     tot_share_not_assigned = sum(tech_not_assigned_by_p.values())
-    for tech, share_by in tech_not_assigned_by_p.items():
 
-        if tot_share_not_assigned == 0:
-            tech_not_assigned_by_p[tech] = 0
-        else:
+    # If no fuel is assigned to enduse, do not change switches
+    if tot_share_not_assigned == 0:
+
+        # Add zero share for all technologies not assigned if sums up to 100%
+        zero_share = 0
+        for tech in tech_not_assigned_by_p:
+            switch_new = read_data.ServiceSwitch(
+                enduse=enduse,
+                sector=sector,
+                technology_install=tech,
+                service_share_ey=zero_share,
+                switch_yr=switch_yr)
+            enduse_switches.append(switch_new)
+
+        service_switches_out = enduse_switches
+    else:
+        for tech, share_by in tech_not_assigned_by_p.items():
             tech_not_assigned_by_p[tech] = share_by / tot_share_not_assigned
 
-    # Get all defined technologies in base year
-    for tech in specified_tech_enduse_by[enduse]:
+        # Get all defined technologies in base year
+        for tech in tech_enduse_by:
+            if tech not in switch_technologies:
+                if s_tot_defined == 1.0:
+                    # All technologies are fully defined
+                    switch_new = read_data.ServiceSwitch(
+                        enduse=enduse,
+                        sector=sector,
+                        technology_install=tech,
+                        service_share_ey=0,
+                        switch_yr=switch_yr)
+                else:
+                    # Calculate not defined share in switches
+                    s_not_assigned = 1 - s_tot_defined
 
-        if tech not in switch_technologies:
+                    # Reduce not assigned share proportionally
+                    tech_ey_p = tech_not_assigned_by_p[tech] * s_not_assigned
 
-            if s_tot_defined == 1.0:
-                # All technologies are fully defined
-                switch_new = read_data.ServiceSwitch(
-                    enduse=enduse,
-                    sector=sector,
-                    technology_install=tech,
-                    service_share_ey=0,
-                    switch_yr=switch_yr)
+                    switch_new = read_data.ServiceSwitch(
+                        enduse=enduse,
+                        sector=sector,
+                        technology_install=tech,
+                        service_share_ey=tech_ey_p,
+                        switch_yr=switch_yr)
+
+                service_switches_out.append(switch_new)
             else:
-                # Calculate not defined share in switches
-                s_not_assigned = 1 - s_tot_defined
+                # If assigned copy to final switches
+                for switch_new in enduse_switches:
+                    if switch_new.technology_install == tech:
 
-                # Reduce not assigned share proportionally
-                tech_ey_p = tech_not_assigned_by_p[tech] * s_not_assigned
+                        # If no fuel demand, also no switch possible
+                        if tot_share_not_assigned == 0:
+                            switch_new.service_share_ey = 0
 
-                switch_new = read_data.ServiceSwitch(
-                    enduse=enduse,
-                    sector=sector,
-                    technology_install=tech,
-                    service_share_ey=tech_ey_p,
-                    switch_yr=switch_yr)
-
-            service_switches_out.append(switch_new)
-        else:
-            # If assigned copy to final switches
-            for switch_new in enduse_switches:
-                if switch_new.technology_install == tech:
-
-                    # If no fuel demand, also no switch possible
-                    if tot_share_not_assigned == 0:
-                        switch_new.service_share_ey = 0
-
-                    service_switches_out.append(switch_new)
+                        service_switches_out.append(switch_new)
 
     return service_switches_out
 
@@ -314,12 +322,13 @@ def autocomplete_switches(
         Criteria wheter regionally specific or not
     regions : list
         Regions
-    f_diffusion : 
-        
-    techs_affected_spatial_f : 
+    f_diffusion :
+        Diffusion factors
+    techs_affected_spatial_f : list
+        Technologies affected by spatial diffusion
+    service_switches_from_capacity : list
+        Transformed capacity swithces into service switches
 
-    service_switches_from_capacity : list, default=[]
-        Service switches stemming from capacity switches
     Returns
     -------
     reg_share_s_tech_ey_p : dict
@@ -343,12 +352,11 @@ def autocomplete_switches(
 
     # Iterate enduses
     for enduse in switch_enduses:
-        logging.info("... calculating service switches: {} crit_all_the_same: {}".format(enduse, crit_all_the_same))
+        logging.debug("... calculating service switches: {} crit_all_the_same: {}".format(enduse, crit_all_the_same))
 
         sectors_of_enduse = get_sectors_of_enduse(
             enduse, enduses, sectors)
 
-        # Iterate sectors of enduse
         for sector in sectors_of_enduse:
 
             switches_enduse = init_scripts.get_sector__enduse_switches(
@@ -371,7 +379,6 @@ def autocomplete_switches(
 
                 # Execut function for every narrative year
                 for switch_yr in switch_yrs:
-
                     enduse_switches = []
                     s_tot_defined = 0
                     switch_technologies = []
@@ -387,7 +394,7 @@ def autocomplete_switches(
                         enduse=enduse,
                         s_tech_by_p=s_tech_by_p,
                         switch_technologies=switch_technologies,
-                        specified_tech_enduse_by=specified_tech_enduse_by,
+                        tech_enduse_by=specified_tech_enduse_by[enduse][sector],
                         enduse_switches=enduse_switches,
                         s_tot_defined=s_tot_defined,
                         sector=sector,
@@ -435,7 +442,7 @@ def autocomplete_switches(
                                     if s_tot_defined + s_share_ey_regional > 1.0:
 
                                         if round(s_tot_defined + s_share_ey_regional) > 1:
-                                            # TODO
+                                            # TODO TODO TODO
                                             logging.warning(
                                                 "{}  {} {}".format(s_tot_defined, s_share_ey_regional, s_tot_defined + s_share_ey_regional))
                                             raise Exception(
@@ -471,8 +478,7 @@ def autocomplete_switches(
 
             # Calculate fraction of service for each technology
             reg_share_s_tech_ey_p[sector] = get_share_s_tech_ey(
-                service_switches_out[sector],
-                specified_tech_enduse_by)
+                service_switches_out[sector])
 
     return reg_share_s_tech_ey_p
 
